
from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .oftp_partner_info import OftpPartnerInfo
from .oftp_local_info import OftpLocalInfo


@JsonMap(
    {
        "default_partner_info": "defaultPartnerInfo",
        "my_company_info": "myCompanyInfo",
        "my_partner_info": "myPartnerInfo",
    }
)
class OftpPartnerGroupType(BaseModel):
    """OftpPartnerGroupType

    :param default_partner_info: default_partner_info, defaults to None
    :type default_partner_info: OftpPartnerInfo, optional
    :param my_company_info: my_company_info
    :type my_company_info: OftpLocalInfo
    :param my_partner_info: my_partner_info
    :type my_partner_info: OftpPartnerInfo
    """

    def __init__(
        self,
        my_company_info: OftpLocalInfo,
        my_partner_info: OftpPartnerInfo,
        default_partner_info: OftpPartnerInfo = SENTINEL,
        **kwargs,
    ):
        """OftpPartnerGroupType

        :param default_partner_info: default_partner_info, defaults to None
        :type default_partner_info: OftpPartnerInfo, optional
        :param my_company_info: my_company_info
        :type my_company_info: OftpLocalInfo
        :param my_partner_info: my_partner_info
        :type my_partner_info: OftpPartnerInfo
        """
        if default_partner_info is not SENTINEL:
            self.default_partner_info = self._define_object(
                default_partner_info, OftpPartnerInfo
            )
        self.my_company_info = self._define_object(my_company_info, OftpLocalInfo)
        self.my_partner_info = self._define_object(my_partner_info, OftpPartnerInfo)
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"connector_type": "connectorType", "listener_id": "listenerId"})
class ListenerStatus(BaseModel):
    """ListenerStatus

    :param connector_type: The internal and unique identifier for connector type, which resembles the connector type names presented on the **Build** tab of the user interface., defaults to None
    :type connector_type: str, optional
    :param listener_id: The Component ID for the listener process.
    :type listener_id: str
    :param status: The status of the listener as `listening`, `paused`, or `errored`.
    :type status: str
    """

    def __init__(
        self, listener_id: str, status: str, connector_type: str = SENTINEL, **kwargs
    ):
        """ListenerStatus

        :param connector_type: The internal and unique identifier for connector type, which resembles the connector type names presented on the **Build** tab of the user interface., defaults to None
        :type connector_type: str, optional
        :param listener_id: The Component ID for the listener process.
        :type listener_id: str
        :param status: The status of the listener as `listening`, `paused`, or `errored`.
        :type status: str
        """
        if connector_type is not SENTINEL:
            self.connector_type = connector_type
        self.listener_id = listener_id
        self.status = status
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .execution_summary_record_simple_expression import (
    ExecutionSummaryRecordSimpleExpression,
)
from .execution_summary_record_grouping_expression import (
    ExecutionSummaryRecordGroupingExpression,
)


class ExecutionSummaryRecordExpressionGuard(OneOfBaseModel):
    class_list = {
        "ExecutionSummaryRecordSimpleExpression": ExecutionSummaryRecordSimpleExpression,
        "ExecutionSummaryRecordGroupingExpression": ExecutionSummaryRecordGroupingExpression,
    }


ExecutionSummaryRecordExpression = Union[
    ExecutionSummaryRecordSimpleExpression, ExecutionSummaryRecordGroupingExpression
]




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .execution_connector_expression import (
    ExecutionConnectorExpression,
    ExecutionConnectorExpressionGuard,
)
from .execution_connector_simple_expression import ExecutionConnectorSimpleExpression
from .execution_connector_grouping_expression import (
    ExecutionConnectorGroupingExpression,
)


@JsonMap({})
class ExecutionConnectorQueryConfigQueryFilter(BaseModel):
    """ExecutionConnectorQueryConfigQueryFilter

    :param expression: expression
    :type expression: ExecutionConnectorExpression
    """

    def __init__(self, expression: ExecutionConnectorExpression, **kwargs):
        """ExecutionConnectorQueryConfigQueryFilter

        :param expression: expression
        :type expression: ExecutionConnectorExpression
        """
        self.expression = ExecutionConnectorExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ExecutionConnectorQueryConfig(BaseModel):
    """ExecutionConnectorQueryConfig

    :param query_filter: query_filter
    :type query_filter: ExecutionConnectorQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: ExecutionConnectorQueryConfigQueryFilter, **kwargs
    ):
        """ExecutionConnectorQueryConfig

        :param query_filter: query_filter
        :type query_filter: ExecutionConnectorQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ExecutionConnectorQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .document_count_account_simple_expression import (
    DocumentCountAccountSimpleExpression,
)
from .document_count_account_grouping_expression import (
    DocumentCountAccountGroupingExpression,
)


class DocumentCountAccountExpressionGuard(OneOfBaseModel):
    class_list = {
        "DocumentCountAccountSimpleExpression": DocumentCountAccountSimpleExpression,
        "DocumentCountAccountGroupingExpression": DocumentCountAccountGroupingExpression,
    }


DocumentCountAccountExpression = Union[
    DocumentCountAccountSimpleExpression, DocumentCountAccountGroupingExpression
]




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .process_properties import ProcessProperties


@JsonMap({"process_properties": "ProcessProperties", "process_id": "processId"})
class DeployedProcess(BaseModel):
    """DeployedProcess

    :param process_properties: The complete list of Persisted Process properties within the specified Runtime, Runtime cluster, or cloud, where the definition of each property is by its name and value.
    :type process_properties: ProcessProperties
    :param process_id: A unique ID assigned by the system to the process.
    :type process_id: str
    """

    def __init__(
        self, process_properties: ProcessProperties, process_id: str, **kwargs
    ):
        """DeployedProcess

        :param process_properties: The complete list of Persisted Process properties within the specified Runtime, Runtime cluster, or cloud, where the definition of each property is by its name and value.
        :type process_properties: ProcessProperties
        :param process_id: A unique ID assigned by the system to the process.
        :type process_id: str
        """
        self.process_properties = self._define_object(
            process_properties, ProcessProperties
        )
        self.process_id = process_id
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .browse_field import BrowseField


@JsonMap({"browse_fields": "BrowseFields", "session_id": "sessionId"})
class MapExtensionBrowse(BaseModel):
    """MapExtensionBrowse

    :param browse_fields: browse_fields, defaults to None
    :type browse_fields: List[BrowseField], optional
    :param session_id: session_id, defaults to None
    :type session_id: str, optional
    """

    def __init__(
        self,
        browse_fields: List[BrowseField] = SENTINEL,
        session_id: str = SENTINEL,
        **kwargs,
    ):
        """MapExtensionBrowse

        :param browse_fields: browse_fields, defaults to None
        :type browse_fields: List[BrowseField], optional
        :param session_id: session_id, defaults to None
        :type session_id: str, optional
        """
        if browse_fields is not SENTINEL:
            self.browse_fields = self._define_list(browse_fields, BrowseField)
        if session_id is not SENTINEL:
            self.session_id = session_id
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    AccountUserRole,
    AccountUserRoleQueryConfig,
    AccountUserRoleQueryResponse,
)


class AccountUserRoleService(BaseService):

    @cast_models
    def create_account_user_role(
        self, request_body: AccountUserRole = None
    ) -> Union[AccountUserRole, str]:
        """Adds a user to an account. If you provide a user ID (email address) that does not exist, the system creates the user and adds them to the account.

         When creating a new user, the API request does not require the firstName and lastName fields. If you do not provide those fields, it assigns the default firstName or lastName values automatically. If you include the firstName and lastName fields in a CREATE request for a user name that exists, the request ignores those fields.

        :param request_body: The request body., defaults to None
        :type request_body: AccountUserRole, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountUserRole, str]
        """

        Validator(AccountUserRole).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountUserRole",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountUserRole._unmap(response)
        if content == "application/xml":
            return AccountUserRole._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_account_user_role(
        self, request_body: AccountUserRoleQueryConfig = None
    ) -> Union[AccountUserRoleQueryResponse, str]:
        """For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: AccountUserRoleQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountUserRoleQueryResponse, str]
        """

        Validator(AccountUserRoleQueryConfig).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountUserRole/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountUserRoleQueryResponse._unmap(response)
        if content == "application/xml":
            return AccountUserRoleQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_account_user_role(
        self, request_body: str
    ) -> Union[AccountUserRoleQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountUserRoleQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountUserRole/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountUserRoleQueryResponse._unmap(response)
        if content == "application/xml":
            return AccountUserRoleQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_account_user_role(self, id_: str) -> None:
        """Removes the specified user by a specified conceptual Account User Role object ID from an account.

        :param id_: id_
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountUserRole/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..atom_counters import AtomCountersService
from ...models import AtomCounters


class AtomCountersServiceAsync(AtomCountersService):
    """
    Async Wrapper for AtomCountersServiceAsync
    """

    def update_atom_counters(
        self, id_: str, request_body: AtomCounters = None
    ) -> Awaitable[Union[AtomCounters, str]]:
        return to_async(super().update_atom_counters)(id_, request_body)




from typing import Dict
from base64 import b64encode

from .base_header import BaseHeader


class BasicAuth(BaseHeader):
    """
    A class for handling Basic authentication in headers.

    :ivar str _username: The username for Basic authentication.
    :ivar str _password: The password for Basic authentication.
    """

    _username = ""
    _password = ""

    def __init__(self, username: str, password: str):
        """
        Initialize the BasicAuth instance.

        :param username: The username for Basic authentication.
        :type username: str
        :param password: The password for Basic authentication.
        :type password: str
        """
        self._username = username
        self._password = password

    def set_value(self, value: dict[str, str]) -> None:
        """
        Set the username and password for Basic authentication.

        :param value: A dictionary with keys 'username' and 'password'.
        :type value: dict[str, str]
        """
        self._username = value.get("username")
        self._password = value.get("password")

    def get_headers(self) -> Dict[str, str]:
        """
        Get the headers with the Authorization field set to the Basic authentication token.

        :return: A dictionary with the Authorization field set to the Basic authentication token.
        :rtype: Dict[str, str]
        """
        token = (
            "Basic " + b64encode(f"{self._username}:{self._password}".encode()).decode()
        )
        return {"Authorization": token}




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    AccountCloudAttachmentQuota,
    AccountCloudAttachmentQuotaBulkRequest,
    AccountCloudAttachmentQuotaBulkResponse,
)


class AccountCloudAttachmentQuotaService(BaseService):

    @cast_models
    def create_account_cloud_attachment_quota(
        self, request_body: AccountCloudAttachmentQuota = None
    ) -> Union[AccountCloudAttachmentQuota, str]:
        """- Use the CREATE operation to create a new Cloud quota and determine the maximum number of Runtime attachments that you can create on the account.
         - You can use the CREATE or UPDATE operations interchangeably to edit a Cloud quota value. Both operations can act as an update after creating the quota.
         - CREATE and UPDATE use the same REST endpoint, as seen in the next section of sample code REST calls. When calling the endpoint for an account that has a quota set, the endpoint acts as an update and modifies the existing value, as explained in the previous item. When calling the endpoint for an account that does not already have a quota set, the endpoint creates a new quota.
         - You cannot set the Cloud quota less than the number of attachments that currently exist on the account, unless you are setting the value to -1 for unlimited. Attempting to do so returns an error.
         - The CREATE operation returns an id value that you can use in a GET operation to retrieve the existing quota for a specific account's Cloud ID.

        :param request_body: The request body., defaults to None
        :type request_body: AccountCloudAttachmentQuota, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountCloudAttachmentQuota, str]
        """

        Validator(AccountCloudAttachmentQuota).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountCloudAttachmentQuota",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountCloudAttachmentQuota._unmap(response)
        if content == "application/xml":
            return AccountCloudAttachmentQuota._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_account_cloud_attachment_quota(
        self, id_: str
    ) -> Union[AccountCloudAttachmentQuota, str]:
        """Retrieves the Cloud quota value currently existing for a Cloud ID on a specific account. The GET operation requires an additional ID (id), and differs from the `cloudId` and `accountId`.

        :param id_: A unique ID generated for a newly created or recently updated Cloud quota (using the Account Cloud Attachment quota object). You can use this ID to get a Cloud quota for an account's specific Cloud ID.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountCloudAttachmentQuota, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountCloudAttachmentQuota/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountCloudAttachmentQuota._unmap(response)
        if content == "application/xml":
            return AccountCloudAttachmentQuota._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_account_cloud_attachment_quota(
        self, id_: str, request_body: AccountCloudAttachmentQuota = None
    ) -> Union[AccountCloudAttachmentQuota, str]:
        """Edit the number of Runtime attachments that you can create on the given account. Specify the IDs of both the account and the Runtime cloud that you want to update. You cannot set the Cloud quota less than the number of attachments that currently exist on the account, unless you are setting the value to -1 for unlimited. Attempting to do so returns an error.

        :param request_body: The request body., defaults to None
        :type request_body: AccountCloudAttachmentQuota, optional
        :param id_: A unique ID generated for a newly created or recently updated Cloud quota (using the Account Cloud Attachment quota object). You can use this ID to get a Cloud quota for an account's specific Cloud ID.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountCloudAttachmentQuota, str]
        """

        Validator(AccountCloudAttachmentQuota).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountCloudAttachmentQuota/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountCloudAttachmentQuota._unmap(response)
        if content == "application/xml":
            return AccountCloudAttachmentQuota._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_account_cloud_attachment_quota(self, id_: str) -> None:
        """Deletes a Cloud quota for a given account.

        :param id_: A unique ID generated for a newly created or recently updated Cloud quota (using the Account Cloud Attachment quota object). You can use this ID to get a Cloud quota for an account's specific Cloud ID.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountCloudAttachmentQuota/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_account_cloud_attachment_quota(
        self, request_body: AccountCloudAttachmentQuotaBulkRequest = None
    ) -> Union[AccountCloudAttachmentQuotaBulkResponse, str]:
        """To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: AccountCloudAttachmentQuotaBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountCloudAttachmentQuotaBulkResponse, str]
        """

        Validator(AccountCloudAttachmentQuotaBulkRequest).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountCloudAttachmentQuota/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountCloudAttachmentQuotaBulkResponse._unmap(response)
        if content == "application/xml":
            return AccountCloudAttachmentQuotaBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "cacerts_path": "cacertsPath",
        "external_jdk_path": "externalJDKPath",
        "migrate_certificate": "migrateCertificate",
        "pref_jre_location": "prefJreLocation",
    }
)
class JavaUpgradeOptions(BaseModel):
    """JavaUpgradeOptions

    :param cacerts_path: (Optional) If specified, defines the directory file path from which to migrate custom certificates.If you include this optional attribute in your request, the directory file path in your local drive must contain either of the following folder patterns jre/lib/security/cacerts, or lib/security/cacerts , wherecacerts is the name of the certificates file.\<br /\>**Note:** You must title the certificate file as cacerts.\<br /\> \<br /\> Because the Upgrade Java API operationautomatically appends the lib/security and jre/lib/security part of the file path to search for the cacerts file name, you need only to supply the root directory OR the root directory and JRE folder in the cacerts value of your request. For example, if your certificates file is located at /boomi_atom/jre/lib/security/cacerts, you need to specify either /boomi_atom/jre or /boomi_atom in the attribute's value.\<br /\> \<br /\>If you omit this attribute from the request or it does not populate its value, it uses the default JVM directory. The migrateCertificates value must be set to true for the override to take place.
    :type cacerts_path: str
    :param external_jdk_path: (Required if you set `prefJreLocation` to `external`) Defines the directory file path to find the custom JDK your container uses after the upgrade. Include this attribute in the request when you want to use a custom JDK rather than one provided by [Boomi](lib-Boomi_Keywords_0346af2b-13d7-491e-bec9-18c5d89225bf.md#BOOMI_DELL). If you do not specify this attribute, the upgrader automatically installs a new [Boomi](lib-Boomi_Keywords_0346af2b-13d7-491e-bec9-18c5d89225bf.md#BOOMI_DELL)-provided JRE to your local directory with the latest supported version of Java.
    :type external_jdk_path: str
    :param migrate_certificate: (Optional) If set to true in the request body, custom certificate migration occurs automatically to the new version of Java. It places certificates in a new JVM of the same directory where you originally installed the container.If set to false in the request body, custom certificates do not migrate.\<br /\>**Note:** If you include `JavaUpgradeOptions` but omit `migrateCertificate` from the request, the response automatically sets the `migrateCertificate` field to false., defaults to None
    :type migrate_certificate: bool, optional
    :param pref_jre_location: Determines how the installation's JRE is updated or whether to use an external JDK. The following values are possible:-   Switch the preferred JRE to the installation's JRE directory \<br /\>- internal\<br /\> -   Do not make changes to the preferred JRE - current\<br /\>- Use external JDK for preferred JRE - external\<br /\>If you set `prefJreLocation` to external you must also specify `externalJDKPath`.\<br /\>If you do not specify a value or omit the parameter, the default value is current.
    :type pref_jre_location: str
    """

    def __init__(
        self,
        cacerts_path: str,
        external_jdk_path: str,
        pref_jre_location: str,
        migrate_certificate: bool = SENTINEL,
        **kwargs
    ):
        """JavaUpgradeOptions

        :param cacerts_path: (Optional) If specified, defines the directory file path from which to migrate custom certificates.If you include this optional attribute in your request, the directory file path in your local drive must contain either of the following folder patterns jre/lib/security/cacerts, or lib/security/cacerts , wherecacerts is the name of the certificates file.\<br /\>**Note:** You must title the certificate file as cacerts.\<br /\> \<br /\> Because the Upgrade Java API operationautomatically appends the lib/security and jre/lib/security part of the file path to search for the cacerts file name, you need only to supply the root directory OR the root directory and JRE folder in the cacerts value of your request. For example, if your certificates file is located at /boomi_atom/jre/lib/security/cacerts, you need to specify either /boomi_atom/jre or /boomi_atom in the attribute's value.\<br /\> \<br /\>If you omit this attribute from the request or it does not populate its value, it uses the default JVM directory. The migrateCertificates value must be set to true for the override to take place.
        :type cacerts_path: str
        :param external_jdk_path: (Required if you set `prefJreLocation` to `external`) Defines the directory file path to find the custom JDK your container uses after the upgrade. Include this attribute in the request when you want to use a custom JDK rather than one provided by [Boomi](lib-Boomi_Keywords_0346af2b-13d7-491e-bec9-18c5d89225bf.md#BOOMI_DELL). If you do not specify this attribute, the upgrader automatically installs a new [Boomi](lib-Boomi_Keywords_0346af2b-13d7-491e-bec9-18c5d89225bf.md#BOOMI_DELL)-provided JRE to your local directory with the latest supported version of Java.
        :type external_jdk_path: str
        :param migrate_certificate: (Optional) If set to true in the request body, custom certificate migration occurs automatically to the new version of Java. It places certificates in a new JVM of the same directory where you originally installed the container.If set to false in the request body, custom certificates do not migrate.\<br /\>**Note:** If you include `JavaUpgradeOptions` but omit `migrateCertificate` from the request, the response automatically sets the `migrateCertificate` field to false., defaults to None
        :type migrate_certificate: bool, optional
        :param pref_jre_location: Determines how the installation's JRE is updated or whether to use an external JDK. The following values are possible:-   Switch the preferred JRE to the installation's JRE directory \<br /\>- internal\<br /\> -   Do not make changes to the preferred JRE - current\<br /\>- Use external JDK for preferred JRE - external\<br /\>If you set `prefJreLocation` to external you must also specify `externalJDKPath`.\<br /\>If you do not specify a value or omit the parameter, the default value is current.
        :type pref_jre_location: str
        """
        self.cacerts_path = cacerts_path
        self.external_jdk_path = external_jdk_path
        if migrate_certificate is not SENTINEL:
            self.migrate_certificate = migrate_certificate
        self.pref_jre_location = pref_jre_location
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .component_reference_expression import (
    ComponentReferenceExpression,
    ComponentReferenceExpressionGuard,
)
from .component_reference_simple_expression import ComponentReferenceSimpleExpression
from .component_reference_grouping_expression import (
    ComponentReferenceGroupingExpression,
)


@JsonMap({})
class ComponentReferenceQueryConfigQueryFilter(BaseModel):
    """ComponentReferenceQueryConfigQueryFilter

    :param expression: expression
    :type expression: ComponentReferenceExpression
    """

    def __init__(self, expression: ComponentReferenceExpression, **kwargs):
        """ComponentReferenceQueryConfigQueryFilter

        :param expression: expression
        :type expression: ComponentReferenceExpression
        """
        self.expression = ComponentReferenceExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ComponentReferenceQueryConfig(BaseModel):
    """ComponentReferenceQueryConfig

    :param query_filter: query_filter
    :type query_filter: ComponentReferenceQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: ComponentReferenceQueryConfigQueryFilter, **kwargs
    ):
        """ComponentReferenceQueryConfig

        :param query_filter: query_filter
        :type query_filter: ComponentReferenceQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ComponentReferenceQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .account_group_account_simple_expression import AccountGroupAccountSimpleExpression
from .account_group_account_grouping_expression import (
    AccountGroupAccountGroupingExpression,
)


class AccountGroupAccountExpressionGuard(OneOfBaseModel):
    class_list = {
        "AccountGroupAccountSimpleExpression": AccountGroupAccountSimpleExpression,
        "AccountGroupAccountGroupingExpression": AccountGroupAccountGroupingExpression,
    }


AccountGroupAccountExpression = Union[
    AccountGroupAccountSimpleExpression, AccountGroupAccountGroupingExpression
]




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    AccountGroupIntegrationPack,
    AccountGroupIntegrationPackBulkRequest,
    AccountGroupIntegrationPackBulkResponse,
    AccountGroupIntegrationPackQueryConfig,
    AccountGroupIntegrationPackQueryResponse,
)


class AccountGroupIntegrationPackService(BaseService):

    @cast_models
    def create_account_group_integration_pack(
        self, request_body: AccountGroupIntegrationPack = None
    ) -> Union[AccountGroupIntegrationPack, str]:
        """Adds an integration pack to the requesting account group.

        :param request_body: The request body., defaults to None
        :type request_body: AccountGroupIntegrationPack, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountGroupIntegrationPack, str]
        """

        Validator(AccountGroupIntegrationPack).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountGroupIntegrationPack",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountGroupIntegrationPack._unmap(response)
        if content == "application/xml":
            return AccountGroupIntegrationPack._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_account_group_integration_pack(
        self, id_: str
    ) -> Union[AccountGroupIntegrationPack, str]:
        """The ordinary GET operation retrieves the properties of the AccountGroupIntegrationPack with the specified ID.
        The bulk GET operation retrieves the properties of the AccountGroupIntegrationPack with the specified IDs to a maximum of 100.
        You can obtain AccountGroupIntegrationPack IDs from the QUERY operation.

        :param id_: A unique ID assigned by the system to the integration pack.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountGroupIntegrationPack, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountGroupIntegrationPack/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountGroupIntegrationPack._unmap(response)
        if content == "application/xml":
            return AccountGroupIntegrationPack._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_account_group_integration_pack(self, id_: str) -> None:
        """Removes the integration pack with a specified ID from the requesting account group.
        You can obtain this ID from a QUERY operation.

        :param id_: A unique ID assigned by the system to the integration pack.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountGroupIntegrationPack/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_account_group_integration_pack(
        self, request_body: AccountGroupIntegrationPackBulkRequest = None
    ) -> Union[AccountGroupIntegrationPackBulkResponse, str]:
        """The bulk GET operation returns multiple objects based on the supplied account IDs, to a maximum of 100. To learn more about `bulk`, refer to [Bulk GET operations](#section/Introduction/Bulk-GET-operations).

        :param request_body: The request body., defaults to None
        :type request_body: AccountGroupIntegrationPackBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountGroupIntegrationPackBulkResponse, str]
        """

        Validator(AccountGroupIntegrationPackBulkRequest).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountGroupIntegrationPack/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountGroupIntegrationPackBulkResponse._unmap(response)
        if content == "application/xml":
            return AccountGroupIntegrationPackBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_account_group_integration_pack(
        self, request_body: AccountGroupIntegrationPackQueryConfig = None
    ) -> Union[AccountGroupIntegrationPackQueryResponse, str]:
        """Retrieves all integration packs available to the requesting account group ID.

         For general information about the structure of QUERY filters and how to handle paged results, see the Query filters and Query paging topics.

        :param request_body: The request body., defaults to None
        :type request_body: AccountGroupIntegrationPackQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountGroupIntegrationPackQueryResponse, str]
        """

        Validator(AccountGroupIntegrationPackQueryConfig).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountGroupIntegrationPack/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountGroupIntegrationPackQueryResponse._unmap(response)
        if content == "application/xml":
            return AccountGroupIntegrationPackQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_account_group_integration_pack(
        self, request_body: str
    ) -> Union[AccountGroupIntegrationPackQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AccountGroupIntegrationPackQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/AccountGroupIntegrationPack/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AccountGroupIntegrationPackQueryResponse._unmap(response)
        if content == "application/xml":
            return AccountGroupIntegrationPackQueryResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .account_group_user_role_simple_expression import (
    AccountGroupUserRoleSimpleExpression,
)
from .account_group_user_role_grouping_expression import (
    AccountGroupUserRoleGroupingExpression,
)


class AccountGroupUserRoleExpressionGuard(OneOfBaseModel):
    class_list = {
        "AccountGroupUserRoleSimpleExpression": AccountGroupUserRoleSimpleExpression,
        "AccountGroupUserRoleGroupingExpression": AccountGroupUserRoleGroupingExpression,
    }


AccountGroupUserRoleExpression = Union[
    AccountGroupUserRoleSimpleExpression, AccountGroupUserRoleGroupingExpression
]




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"status_code": "statusCode"})
class ConnectionLicensingDownload(BaseModel):
    """ConnectionLicensingDownload

    :param url: URL to view or download the deployed connection details., defaults to None
    :type url: str, optional
    :param message: message, defaults to None
    :type message: str, optional
    :param status_code: status_code, defaults to None
    :type status_code: str, optional
    """

    def __init__(
        self,
        url: str = SENTINEL,
        message: str = SENTINEL,
        status_code: str = SENTINEL,
        **kwargs
    ):
        """ConnectionLicensingDownload

        :param url: URL to view or download the deployed connection details., defaults to None
        :type url: str, optional
        :param message: message, defaults to None
        :type message: str, optional
        :param status_code: status_code, defaults to None
        :type status_code: str, optional
        """
        if url is not SENTINEL:
            self.url = url
        if message is not SENTINEL:
            self.message = message
        if status_code is not SENTINEL:
            self.status_code = status_code
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..models import (
    TradacomsConnectorRecordQueryConfig,
    TradacomsConnectorRecordQueryResponse,
)


class TradacomsConnectorRecordService(BaseService):

    @cast_models
    def query_tradacoms_connector_record(
        self, request_body: TradacomsConnectorRecordQueryConfig = None
    ) -> Union[TradacomsConnectorRecordQueryResponse, str]:
        """- To filter by a custom field, use the format `customFields`. Use the `fieldName` as the filter property where `fieldName` is the element name of the custom field in the record structure. To get a list of the available custom fields see the [Custom Tracked Field](#tag/CustomTrackedField) object.

         - The STARTS_WITH operator only accepts values that do not include spaces.

         For general information about the structure of QUERY filters, their sample payloads, and how to handle the paged results, refer to [Query filters](#section/Introduction/Query-filters) and [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body., defaults to None
        :type request_body: TradacomsConnectorRecordQueryConfig, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradacomsConnectorRecordQueryResponse, str]
        """

        Validator(TradacomsConnectorRecordQueryConfig).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradacomsConnectorRecord/query",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradacomsConnectorRecordQueryResponse._unmap(response)
        if content == "application/xml":
            return TradacomsConnectorRecordQueryResponse._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def query_more_tradacoms_connector_record(
        self, request_body: str
    ) -> Union[TradacomsConnectorRecordQueryResponse, str]:
        """To learn about using `queryMore`, refer to [Query paging](#section/Introduction/Query-paging).

        :param request_body: The request body.
        :type request_body: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[TradacomsConnectorRecordQueryResponse, str]
        """

        Validator(str).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/TradacomsConnectorRecord/queryMore",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body, "text/plain")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return TradacomsConnectorRecordQueryResponse._unmap(response)
        if content == "application/xml":
            return TradacomsConnectorRecordQueryResponse._unmap(response)
        raise ApiError("Error on deserializing the response.", status, response)




from typing import Generator, Optional, Tuple


from .base_handler import BaseHandler
from ....hooks.hook import DefaultHook
from ...transport.request import Request
from ...transport.response import Response
from ...transport.request_error import RequestError
from ...transport.api_error import ApiError


class HookHandler(BaseHandler):
    """
    Handler for calling hooks.

    :ivar Hook _hook: The hook to be called. This is a placeholder and should be replaced with an instance of the actual hook class.
    """

    def __init__(self):
        """
        Initialize a new instance of HookHandler.
        """
        super().__init__()
        self._hook = DefaultHook()

    def handle(
        self, request: Request
    ) -> Tuple[Optional[Response], Optional[Exception]]:
        """
        Call the beforeRequest hook before passing the request to the next handler in the chain.
        Call the afterResponse hook after receiving a response from the next handler in the chain.
        Call the onError hook if an error occurs in the next handler in the chain.

        :param Request request: The request to handle.
        :return: The response and any error that occurred.
        :rtype: Tuple[Optional[Response], Optional[Exception]]
        """
        if self._next_handler is None:
            raise RequestError("Handler chain is incomplete")

        self._hook.before_request(request)
        response, error = self._next_handler.handle(request)
        self._handle_response(request, response, error)

        return response, error

    def stream(
        self, request: Request
    ) -> Generator[Tuple[Optional[Response], Optional[Exception]], None, None]:
        """
        Call the beforeRequest hook before passing the request to the next handler in the chain.
        Call the afterResponse hook after receiving a response from the next handler in the chain.
        Call the onError hook if an error occurs in the next handler in the chain.

        :param Request request: The request to handle.
        :return: The response and any error that occurred.
        :rtype: Generator[Tuple[Optional[Response], Optional[Excpetion]], None, None]
        """
        if self._next_handler is None:
            raise RequestError("Handler chain is incomplete")

        self._hook.before_request(request)
        for response, error in self._next_handler.stream(request):
            self._handle_response(request, response, error)
            yield response, error

    def _handle_response(self, request: Request, response: Response, error: ApiError):
        if error is not None and isinstance(error, ApiError):
            self._hook.on_error(error, request, error.response)
        else:
            self._hook.after_response(request, response)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .merge_request_simple_expression import MergeRequestSimpleExpression
from .merge_request_grouping_expression import MergeRequestGroupingExpression


class MergeRequestExpressionGuard(OneOfBaseModel):
    class_list = {
        "MergeRequestSimpleExpression": MergeRequestSimpleExpression,
        "MergeRequestGroupingExpression": MergeRequestGroupingExpression,
    }


MergeRequestExpression = Union[
    MergeRequestSimpleExpression, MergeRequestGroupingExpression
]




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .doc_cache_key_input import DocCacheKeyInput


@JsonMap({"input": "Input"})
class DocCacheKeyInputs(BaseModel):
    """DocCacheKeyInputs

    :param input: input, defaults to None
    :type input: List[DocCacheKeyInput], optional
    """

    def __init__(self, input: List[DocCacheKeyInput] = SENTINEL, **kwargs):
        """DocCacheKeyInputs

        :param input: input, defaults to None
        :type input: List[DocCacheKeyInput], optional
        """
        if input is not SENTINEL:
            self.input = self._define_list(input, DocCacheKeyInput)
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..custom_tracked_field import CustomTrackedFieldService
from ...models import CustomTrackedFieldQueryResponse, CustomTrackedFieldQueryConfig


class CustomTrackedFieldServiceAsync(CustomTrackedFieldService):
    """
    Async Wrapper for CustomTrackedFieldServiceAsync
    """

    def query_custom_tracked_field(
        self, request_body: CustomTrackedFieldQueryConfig = None
    ) -> Awaitable[Union[CustomTrackedFieldQueryResponse, str]]:
        return to_async(super().query_custom_tracked_field)(request_body)

    def query_more_custom_tracked_field(
        self, request_body: str
    ) -> Awaitable[Union[CustomTrackedFieldQueryResponse, str]]:
        return to_async(super().query_more_custom_tracked_field)(request_body)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .shared_web_server_login_module_configuration import (
    SharedWebServerLoginModuleConfiguration,
)


@JsonMap(
    {
        "auth_type": "authType",
        "cache_authentication_timeout": "cacheAuthenticationTimeout",
        "cache_authorization_credentials": "cacheAuthorizationCredentials",
        "client_certificate_header_name": "clientCertificateHeaderName",
        "login_module_class_name": "loginModuleClassName",
        "login_module_options": "loginModuleOptions",
    }
)
class SharedWebServerAuthentication(BaseModel):
    """SharedWebServerAuthentication

    :param auth_type: auth_type
    :type auth_type: str
    :param cache_authentication_timeout: cache_authentication_timeout, defaults to None
    :type cache_authentication_timeout: int, optional
    :param cache_authorization_credentials: cache_authorization_credentials, defaults to None
    :type cache_authorization_credentials: bool, optional
    :param client_certificate_header_name: client_certificate_header_name
    :type client_certificate_header_name: str
    :param login_module_class_name: login_module_class_name
    :type login_module_class_name: str
    :param login_module_options: login_module_options
    :type login_module_options: SharedWebServerLoginModuleConfiguration
    """

    def __init__(
        self,
        auth_type: str,
        client_certificate_header_name: str,
        login_module_class_name: str,
        login_module_options: SharedWebServerLoginModuleConfiguration,
        cache_authentication_timeout: int = SENTINEL,
        cache_authorization_credentials: bool = SENTINEL,
        **kwargs,
    ):
        """SharedWebServerAuthentication

        :param auth_type: auth_type
        :type auth_type: str
        :param cache_authentication_timeout: cache_authentication_timeout, defaults to None
        :type cache_authentication_timeout: int, optional
        :param cache_authorization_credentials: cache_authorization_credentials, defaults to None
        :type cache_authorization_credentials: bool, optional
        :param client_certificate_header_name: client_certificate_header_name
        :type client_certificate_header_name: str
        :param login_module_class_name: login_module_class_name
        :type login_module_class_name: str
        :param login_module_options: login_module_options
        :type login_module_options: SharedWebServerLoginModuleConfiguration
        """
        self.auth_type = auth_type
        if cache_authentication_timeout is not SENTINEL:
            self.cache_authentication_timeout = cache_authentication_timeout
        if cache_authorization_credentials is not SENTINEL:
            self.cache_authorization_credentials = cache_authorization_credentials
        self.client_certificate_header_name = client_certificate_header_name
        self.login_module_class_name = login_module_class_name
        self.login_module_options = self._define_object(
            login_module_options, SharedWebServerLoginModuleConfiguration
        )
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    PersistedProcessProperties,
    AsyncOperationTokenResult,
    PersistedProcessPropertiesAsyncResponse
)


class PersistedProcessPropertiesService(BaseService):

    @cast_models
    def update_persisted_process_properties(
        self, id_: str, request_body: PersistedProcessProperties = None
    ) -> Union[PersistedProcessProperties, str]:
        """The UPDATE operation updates Persisted Process Property values for the specified Runtime. Using the UPDATE operation overrides all current property settings. Therefore, strongly recommends that you include a complete list of all Persisted Process properties you want to keep or update. If you do not list a current persisted process property in the Persisted Process properties object, the UPDATE operation deletes those properties.

        >**Note:** You can update the Persisted Process properties if you have either the Runtime Management privilege or the Runtime Management Read Access, along with the Persisted Process Property Read and Write Access privilege.

        :param request_body: The request body., defaults to None
        :type request_body: PersistedProcessProperties, optional
        :param id_: A unique ID assigned by the system to the Runtime.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PersistedProcessProperties, str]
        """

        Validator(PersistedProcessProperties).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/PersistedProcessProperties/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PersistedProcessProperties._unmap(response)
        if content == "application/xml":
            return PersistedProcessProperties._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_get_persisted_process_properties(
        self, id_: str
    ) -> Union[AsyncOperationTokenResult, str]:
        """The GET operation returns the current state of the Persisted Process properties names and values for the specified Runtime.
        The initial GET operation returns a token for the specified Runtime.

        :param id_: A unique ID assigned by the system to the Runtime.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[AsyncOperationTokenResult, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/PersistedProcessProperties/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return AsyncOperationTokenResult._unmap(response)
        if content == "application/xml":
            return AsyncOperationTokenResult._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def async_token_persisted_process_properties(
        self, token: str
    ) -> Union[PersistedProcessPropertiesAsyncResponse, str]:
        """For a response, use the token from the response in a new request.

        :param token: Takes in the token from a previous call to return a result.
        :type token: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[PersistedProcessPropertiesAsyncResponse, str]
        """

        Validator(str).validate(token)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/async/PersistedProcessProperties/response/{{token}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("token", token)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return PersistedProcessPropertiesAsyncResponse._unmap(response)
        if content == "application/xml":
            return PersistedProcessPropertiesAsyncResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .merge_request_expression import (
    MergeRequestExpression,
    MergeRequestExpressionGuard,
)
from .merge_request_simple_expression import MergeRequestSimpleExpression
from .merge_request_grouping_expression import MergeRequestGroupingExpression


@JsonMap({})
class MergeRequestQueryConfigQueryFilter(BaseModel):
    """MergeRequestQueryConfigQueryFilter

    :param expression: expression
    :type expression: MergeRequestExpression
    """

    def __init__(self, expression: MergeRequestExpression, **kwargs):
        """MergeRequestQueryConfigQueryFilter

        :param expression: expression
        :type expression: MergeRequestExpression
        """
        self.expression = MergeRequestExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class MergeRequestQueryConfig(BaseModel):
    """MergeRequestQueryConfig

    :param query_filter: query_filter
    :type query_filter: MergeRequestQueryConfigQueryFilter
    """

    def __init__(self, query_filter: MergeRequestQueryConfigQueryFilter, **kwargs):
        """MergeRequestQueryConfig

        :param query_filter: query_filter
        :type query_filter: MergeRequestQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, MergeRequestQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class UnhControlInfoControllingAgency(Enum):
    """An enumeration representing different categories.

    :cvar AA: "AA"
    :vartype AA: str
    :cvar AB: "AB"
    :vartype AB: str
    :cvar AC: "AC"
    :vartype AC: str
    :cvar AD: "AD"
    :vartype AD: str
    :cvar AE: "AE"
    :vartype AE: str
    :cvar CC: "CC"
    :vartype CC: str
    :cvar CE: "CE"
    :vartype CE: str
    :cvar EC: "EC"
    :vartype EC: str
    :cvar ED: "ED"
    :vartype ED: str
    :cvar EE: "EE"
    :vartype EE: str
    :cvar EN: "EN"
    :vartype EN: str
    :cvar ER: "ER"
    :vartype ER: str
    :cvar EU: "EU"
    :vartype EU: str
    :cvar EX: "EX"
    :vartype EX: str
    :cvar IA: "IA"
    :vartype IA: str
    :cvar KE: "KE"
    :vartype KE: str
    :cvar LI: "LI"
    :vartype LI: str
    :cvar OD: "OD"
    :vartype OD: str
    :cvar RI: "RI"
    :vartype RI: str
    :cvar RT: "RT"
    :vartype RT: str
    :cvar UN: "UN"
    :vartype UN: str
    """

    AA = "AA"
    AB = "AB"
    AC = "AC"
    AD = "AD"
    AE = "AE"
    CC = "CC"
    CE = "CE"
    EC = "EC"
    ED = "ED"
    EE = "EE"
    EN = "EN"
    ER = "ER"
    EU = "EU"
    EX = "EX"
    IA = "IA"
    KE = "KE"
    LI = "LI"
    OD = "OD"
    RI = "RI"
    RT = "RT"
    UN = "UN"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value, UnhControlInfoControllingAgency._member_map_.values()
            )
        )


class UnhControlInfoRelease(Enum):
    """An enumeration representing different categories.

    :cvar EDIFACTRELEASE1: "EDIFACTRELEASE_1"
    :vartype EDIFACTRELEASE1: str
    :cvar EDIFACTRELEASE2: "EDIFACTRELEASE_2"
    :vartype EDIFACTRELEASE2: str
    :cvar EDIFACTRELEASE902: "EDIFACTRELEASE_902"
    :vartype EDIFACTRELEASE902: str
    :cvar EDIFACTRELEASE911: "EDIFACTRELEASE_911"
    :vartype EDIFACTRELEASE911: str
    :cvar EDIFACTRELEASE912: "EDIFACTRELEASE_912"
    :vartype EDIFACTRELEASE912: str
    :cvar EDIFACTRELEASE921: "EDIFACTRELEASE_921"
    :vartype EDIFACTRELEASE921: str
    :cvar EDIFACTRELEASE932: "EDIFACTRELEASE_932"
    :vartype EDIFACTRELEASE932: str
    :cvar EDIFACTRELEASE93A: "EDIFACTRELEASE_93A"
    :vartype EDIFACTRELEASE93A: str
    :cvar EDIFACTRELEASE94A: "EDIFACTRELEASE_94A"
    :vartype EDIFACTRELEASE94A: str
    :cvar EDIFACTRELEASE94B: "EDIFACTRELEASE_94B"
    :vartype EDIFACTRELEASE94B: str
    :cvar EDIFACTRELEASE95A: "EDIFACTRELEASE_95A"
    :vartype EDIFACTRELEASE95A: str
    :cvar EDIFACTRELEASE95B: "EDIFACTRELEASE_95B"
    :vartype EDIFACTRELEASE95B: str
    :cvar EDIFACTRELEASE96A: "EDIFACTRELEASE_96A"
    :vartype EDIFACTRELEASE96A: str
    :cvar EDIFACTRELEASE96B: "EDIFACTRELEASE_96B"
    :vartype EDIFACTRELEASE96B: str
    :cvar EDIFACTRELEASE97A: "EDIFACTRELEASE_97A"
    :vartype EDIFACTRELEASE97A: str
    :cvar EDIFACTRELEASE97B: "EDIFACTRELEASE_97B"
    :vartype EDIFACTRELEASE97B: str
    :cvar EDIFACTRELEASE98A: "EDIFACTRELEASE_98A"
    :vartype EDIFACTRELEASE98A: str
    :cvar EDIFACTRELEASE98B: "EDIFACTRELEASE_98B"
    :vartype EDIFACTRELEASE98B: str
    :cvar EDIFACTRELEASE99A: "EDIFACTRELEASE_99A"
    :vartype EDIFACTRELEASE99A: str
    :cvar EDIFACTRELEASE99B: "EDIFACTRELEASE_99B"
    :vartype EDIFACTRELEASE99B: str
    :cvar EDIFACTRELEASE00A: "EDIFACTRELEASE_00A"
    :vartype EDIFACTRELEASE00A: str
    :cvar EDIFACTRELEASE00B: "EDIFACTRELEASE_00B"
    :vartype EDIFACTRELEASE00B: str
    :cvar EDIFACTRELEASE01A: "EDIFACTRELEASE_01A"
    :vartype EDIFACTRELEASE01A: str
    :cvar EDIFACTRELEASE01B: "EDIFACTRELEASE_01B"
    :vartype EDIFACTRELEASE01B: str
    :cvar EDIFACTRELEASE02A: "EDIFACTRELEASE_02A"
    :vartype EDIFACTRELEASE02A: str
    :cvar EDIFACTRELEASE02B: "EDIFACTRELEASE_02B"
    :vartype EDIFACTRELEASE02B: str
    :cvar EDIFACTRELEASE03A: "EDIFACTRELEASE_03A"
    :vartype EDIFACTRELEASE03A: str
    :cvar EDIFACTRELEASE03B: "EDIFACTRELEASE_03B"
    :vartype EDIFACTRELEASE03B: str
    :cvar EDIFACTRELEASE04A: "EDIFACTRELEASE_04A"
    :vartype EDIFACTRELEASE04A: str
    :cvar EDIFACTRELEASE04B: "EDIFACTRELEASE_04B"
    :vartype EDIFACTRELEASE04B: str
    :cvar EDIFACTRELEASE05A: "EDIFACTRELEASE_05A"
    :vartype EDIFACTRELEASE05A: str
    :cvar EDIFACTRELEASE05B: "EDIFACTRELEASE_05B"
    :vartype EDIFACTRELEASE05B: str
    :cvar EDIFACTRELEASE06A: "EDIFACTRELEASE_06A"
    :vartype EDIFACTRELEASE06A: str
    :cvar EDIFACTRELEASE06B: "EDIFACTRELEASE_06B"
    :vartype EDIFACTRELEASE06B: str
    :cvar EDIFACTRELEASE07A: "EDIFACTRELEASE_07A"
    :vartype EDIFACTRELEASE07A: str
    :cvar EDIFACTRELEASE07B: "EDIFACTRELEASE_07B"
    :vartype EDIFACTRELEASE07B: str
    :cvar EDIFACTRELEASE08A: "EDIFACTRELEASE_08A"
    :vartype EDIFACTRELEASE08A: str
    :cvar EDIFACTRELEASE08B: "EDIFACTRELEASE_08B"
    :vartype EDIFACTRELEASE08B: str
    :cvar EDIFACTRELEASE09A: "EDIFACTRELEASE_09A"
    :vartype EDIFACTRELEASE09A: str
    :cvar EDIFACTRELEASE09B: "EDIFACTRELEASE_09B"
    :vartype EDIFACTRELEASE09B: str
    """

    EDIFACTRELEASE1 = "EDIFACTRELEASE_1"
    EDIFACTRELEASE2 = "EDIFACTRELEASE_2"
    EDIFACTRELEASE902 = "EDIFACTRELEASE_902"
    EDIFACTRELEASE911 = "EDIFACTRELEASE_911"
    EDIFACTRELEASE912 = "EDIFACTRELEASE_912"
    EDIFACTRELEASE921 = "EDIFACTRELEASE_921"
    EDIFACTRELEASE932 = "EDIFACTRELEASE_932"
    EDIFACTRELEASE93A = "EDIFACTRELEASE_93A"
    EDIFACTRELEASE94A = "EDIFACTRELEASE_94A"
    EDIFACTRELEASE94B = "EDIFACTRELEASE_94B"
    EDIFACTRELEASE95A = "EDIFACTRELEASE_95A"
    EDIFACTRELEASE95B = "EDIFACTRELEASE_95B"
    EDIFACTRELEASE96A = "EDIFACTRELEASE_96A"
    EDIFACTRELEASE96B = "EDIFACTRELEASE_96B"
    EDIFACTRELEASE97A = "EDIFACTRELEASE_97A"
    EDIFACTRELEASE97B = "EDIFACTRELEASE_97B"
    EDIFACTRELEASE98A = "EDIFACTRELEASE_98A"
    EDIFACTRELEASE98B = "EDIFACTRELEASE_98B"
    EDIFACTRELEASE99A = "EDIFACTRELEASE_99A"
    EDIFACTRELEASE99B = "EDIFACTRELEASE_99B"
    EDIFACTRELEASE00A = "EDIFACTRELEASE_00A"
    EDIFACTRELEASE00B = "EDIFACTRELEASE_00B"
    EDIFACTRELEASE01A = "EDIFACTRELEASE_01A"
    EDIFACTRELEASE01B = "EDIFACTRELEASE_01B"
    EDIFACTRELEASE02A = "EDIFACTRELEASE_02A"
    EDIFACTRELEASE02B = "EDIFACTRELEASE_02B"
    EDIFACTRELEASE03A = "EDIFACTRELEASE_03A"
    EDIFACTRELEASE03B = "EDIFACTRELEASE_03B"
    EDIFACTRELEASE04A = "EDIFACTRELEASE_04A"
    EDIFACTRELEASE04B = "EDIFACTRELEASE_04B"
    EDIFACTRELEASE05A = "EDIFACTRELEASE_05A"
    EDIFACTRELEASE05B = "EDIFACTRELEASE_05B"
    EDIFACTRELEASE06A = "EDIFACTRELEASE_06A"
    EDIFACTRELEASE06B = "EDIFACTRELEASE_06B"
    EDIFACTRELEASE07A = "EDIFACTRELEASE_07A"
    EDIFACTRELEASE07B = "EDIFACTRELEASE_07B"
    EDIFACTRELEASE08A = "EDIFACTRELEASE_08A"
    EDIFACTRELEASE08B = "EDIFACTRELEASE_08B"
    EDIFACTRELEASE09A = "EDIFACTRELEASE_09A"
    EDIFACTRELEASE09B = "EDIFACTRELEASE_09B"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, UnhControlInfoRelease._member_map_.values()))


class UnhControlInfoVersion(Enum):
    """An enumeration representing different categories.

    :cvar EDIFACTVERSION1: "EDIFACTVERSION_1"
    :vartype EDIFACTVERSION1: str
    :cvar EDIFACTVERSION2: "EDIFACTVERSION_2"
    :vartype EDIFACTVERSION2: str
    :cvar EDIFACTVERSION4: "EDIFACTVERSION_4"
    :vartype EDIFACTVERSION4: str
    :cvar EDIFACTVERSION88: "EDIFACTVERSION_88"
    :vartype EDIFACTVERSION88: str
    :cvar EDIFACTVERSION89: "EDIFACTVERSION_89"
    :vartype EDIFACTVERSION89: str
    :cvar EDIFACTVERSION90: "EDIFACTVERSION_90"
    :vartype EDIFACTVERSION90: str
    :cvar EDIFACTVERSIOND: "EDIFACTVERSION_D"
    :vartype EDIFACTVERSIOND: str
    :cvar EDIFACTVERSIONS: "EDIFACTVERSION_S"
    :vartype EDIFACTVERSIONS: str
    """

    EDIFACTVERSION1 = "EDIFACTVERSION_1"
    EDIFACTVERSION2 = "EDIFACTVERSION_2"
    EDIFACTVERSION4 = "EDIFACTVERSION_4"
    EDIFACTVERSION88 = "EDIFACTVERSION_88"
    EDIFACTVERSION89 = "EDIFACTVERSION_89"
    EDIFACTVERSION90 = "EDIFACTVERSION_90"
    EDIFACTVERSIOND = "EDIFACTVERSION_D"
    EDIFACTVERSIONS = "EDIFACTVERSION_S"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, UnhControlInfoVersion._member_map_.values()))


@JsonMap(
    {
        "assoc_assigned_code": "assocAssignedCode",
        "common_access_ref": "commonAccessRef",
        "controlling_agency": "controllingAgency",
    }
)
class UnhControlInfo(BaseModel):
    """UnhControlInfo

    :param assoc_assigned_code: assoc_assigned_code, defaults to None
    :type assoc_assigned_code: str, optional
    :param common_access_ref: common_access_ref, defaults to None
    :type common_access_ref: str, optional
    :param controlling_agency: controlling_agency, defaults to None
    :type controlling_agency: UnhControlInfoControllingAgency, optional
    :param release: release, defaults to None
    :type release: UnhControlInfoRelease, optional
    :param version: version, defaults to None
    :type version: UnhControlInfoVersion, optional
    """

    def __init__(
        self,
        assoc_assigned_code: str = SENTINEL,
        common_access_ref: str = SENTINEL,
        controlling_agency: UnhControlInfoControllingAgency = SENTINEL,
        release: UnhControlInfoRelease = SENTINEL,
        version: UnhControlInfoVersion = SENTINEL,
        **kwargs
    ):
        """UnhControlInfo

        :param assoc_assigned_code: assoc_assigned_code, defaults to None
        :type assoc_assigned_code: str, optional
        :param common_access_ref: common_access_ref, defaults to None
        :type common_access_ref: str, optional
        :param controlling_agency: controlling_agency, defaults to None
        :type controlling_agency: UnhControlInfoControllingAgency, optional
        :param release: release, defaults to None
        :type release: UnhControlInfoRelease, optional
        :param version: version, defaults to None
        :type version: UnhControlInfoVersion, optional
        """
        if assoc_assigned_code is not SENTINEL:
            self.assoc_assigned_code = assoc_assigned_code
        if common_access_ref is not SENTINEL:
            self.common_access_ref = common_access_ref
        if controlling_agency is not SENTINEL:
            self.controlling_agency = self._enum_matching(
                controlling_agency,
                UnhControlInfoControllingAgency.list(),
                "controlling_agency",
            )
        if release is not SENTINEL:
            self.release = self._enum_matching(
                release, UnhControlInfoRelease.list(), "release"
            )
        if version is not SENTINEL:
            self.version = self._enum_matching(
                version, UnhControlInfoVersion.list(), "version"
            )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({})
class CompDiffAttribute(BaseModel):
    """CompDiffAttribute

    :param idpart: Whether the attribute is part of an identifying key for the element., defaults to None
    :type idpart: bool, optional
    :param ignored: Whether the element described in the `CompDiffElement` section is excluded from the comparative diff results., defaults to None
    :type ignored: bool, optional
    :param name: The name of the attribute for the element that you want to configure for the comparative diff results., defaults to None
    :type name: str, optional
    """

    def __init__(
        self,
        idpart: bool = SENTINEL,
        ignored: bool = SENTINEL,
        name: str = SENTINEL,
        **kwargs
    ):
        """CompDiffAttribute

        :param idpart: Whether the attribute is part of an identifying key for the element., defaults to None
        :type idpart: bool, optional
        :param ignored: Whether the element described in the `CompDiffElement` section is excluded from the comparative diff results., defaults to None
        :type ignored: bool, optional
        :param name: The name of the attribute for the element that you want to configure for the comparative diff results., defaults to None
        :type name: str, optional
        """
        if idpart is not SENTINEL:
            self.idpart = idpart
        if ignored is not SENTINEL:
            self.ignored = ignored
        if name is not SENTINEL:
            self.name = name
        self._kwargs = kwargs




from typing import Union
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..net.transport.api_error import ApiError
from ..net.environment.environment import Environment
from ..models.utils.cast_models import cast_models
from ..net.transport.utils import parse_xml_to_dict
from ..models import (
    RuntimeReleaseSchedule,
    RuntimeReleaseScheduleBulkRequest,
    RuntimeReleaseScheduleBulkResponse,
)


class RuntimeReleaseScheduleService(BaseService):

    @cast_models
    def create_runtime_release_schedule(
        self, request_body: RuntimeReleaseSchedule = None
    ) -> Union[RuntimeReleaseSchedule, str]:
        """The CREATE operation sets a schedule for receiving updates with the scheduleType, dayOfWeek, hourOfDay, and timeZone fields.

        :param request_body: The request body., defaults to None
        :type request_body: RuntimeReleaseSchedule, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[RuntimeReleaseSchedule, str]
        """

        Validator(RuntimeReleaseSchedule).is_optional().validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/RuntimeReleaseSchedule",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return RuntimeReleaseSchedule._unmap(response)
        if content == "application/xml":
            return RuntimeReleaseSchedule._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def get_runtime_release_schedule(
        self, id_: str
    ) -> Union[RuntimeReleaseSchedule, str]:
        """The GET operation returns the current schedule for receiving updates on a specified Runtime, Runtime cluster, or Runtime cloud.

        :param id_: The ID of the container for which you want to set a schedule.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[RuntimeReleaseSchedule, str]
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/RuntimeReleaseSchedule/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("GET")
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return RuntimeReleaseSchedule._unmap(response)
        if content == "application/xml":
            return RuntimeReleaseSchedule._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def update_runtime_release_schedule(
        self, id_: str, request_body: RuntimeReleaseSchedule = None
    ) -> Union[RuntimeReleaseSchedule, str]:
        """The UPDATE operation modifies a set schedule for receiving updates.

        :param request_body: The request body., defaults to None
        :type request_body: RuntimeReleaseSchedule, optional
        :param id_: The ID of the container for which you want to set a schedule.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[RuntimeReleaseSchedule, str]
        """

        Validator(RuntimeReleaseSchedule).is_optional().validate(request_body)
        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/RuntimeReleaseSchedule/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return RuntimeReleaseSchedule._unmap(response)
        if content == "application/xml":
            return RuntimeReleaseSchedule._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)

    @cast_models
    def delete_runtime_release_schedule(self, id_: str) -> None:
        """The DELETE operation sets the scheduleType to NEVER, meaning that the Runtime, Runtime cluster, or Runtime cloud receives updates only during the .

        :param id_: The ID of the container for which you want to set a schedule.
        :type id_: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(id_)

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/RuntimeReleaseSchedule/{{id}}",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .add_path("id", id_)
            .serialize()
            .set_method("DELETE")
        )

        response, status, _ = self.send_request(serialized_request)

    @cast_models
    def bulk_runtime_release_schedule(
        self, request_body: RuntimeReleaseScheduleBulkRequest = None
    ) -> Union[RuntimeReleaseScheduleBulkResponse, str]:
        """bulk_runtime_release_schedule

        :param request_body: The request body., defaults to None
        :type request_body: RuntimeReleaseScheduleBulkRequest, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: The parsed response data.
        :rtype: Union[RuntimeReleaseScheduleBulkResponse, str]
        """

        Validator(RuntimeReleaseScheduleBulkRequest).is_optional().validate(
            request_body
        )

        serialized_request = (
            Serializer(
                f"{self.base_url or Environment.DEFAULT.url}/RuntimeReleaseSchedule/bulk",
                [self.get_access_token(), self.get_basic_auth()],
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response, status, content = self.send_request(serialized_request)
        if content == "application/json":
            return RuntimeReleaseScheduleBulkResponse._unmap(response)
        if content == "application/xml":
            return RuntimeReleaseScheduleBulkResponse._unmap(parse_xml_to_dict(response))
        raise ApiError("Error on deserializing the response.", status, response)




from __future__ import annotations
from typing import Optional
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .deployed_package_expression import (
    DeployedPackageExpression,
    DeployedPackageExpressionGuard,
)
from .deployed_package_simple_expression import DeployedPackageSimpleExpression
from .deployed_package_grouping_expression import DeployedPackageGroupingExpression


@JsonMap({})
class DeployedPackageQueryConfigQueryFilter(BaseModel):
    """DeployedPackageQueryConfigQueryFilter

    :param expression: expression
    :type expression: DeployedPackageExpression
    """

    def __init__(self, expression: DeployedPackageExpression, **kwargs):
        """DeployedPackageQueryConfigQueryFilter

        :param expression: expression
        :type expression: DeployedPackageExpression
        """
        self.expression = DeployedPackageExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class DeployedPackageQueryConfig(BaseModel):
    """DeployedPackageQueryConfig

    :param query_filter: query_filter (optional)
    :type query_filter: Optional[DeployedPackageQueryConfigQueryFilter]
    """

    def __init__(self, query_filter: Optional[DeployedPackageQueryConfigQueryFilter] = None, **kwargs):
        """DeployedPackageQueryConfig

        :param query_filter: query_filter (optional)
        :type query_filter: Optional[DeployedPackageQueryConfigQueryFilter]
        """
        self.query_filter = self._define_object(
            query_filter, DeployedPackageQueryConfigQueryFilter
        ) if query_filter is not None else None
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .edifact_control_info import EdifactControlInfo
from .edifact_options import EdifactOptions


@JsonMap(
    {"edifact_control_info": "EdifactControlInfo", "edifact_options": "EdifactOptions"}
)
class EdifactPartnerInfo(BaseModel):
    """EdifactPartnerInfo

    :param edifact_control_info: edifact_control_info
    :type edifact_control_info: EdifactControlInfo
    :param edifact_options: edifact_options
    :type edifact_options: EdifactOptions
    """

    def __init__(
        self,
        edifact_control_info: EdifactControlInfo,
        edifact_options: EdifactOptions,
        **kwargs,
    ):
        """EdifactPartnerInfo

        :param edifact_control_info: edifact_control_info
        :type edifact_control_info: EdifactControlInfo
        :param edifact_options: edifact_options
        :type edifact_options: EdifactOptions
        """
        self.edifact_control_info = self._define_object(
            edifact_control_info, EdifactControlInfo
        )
        self.edifact_options = self._define_object(edifact_options, EdifactOptions)
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .rosetta_net_control_info import RosettaNetControlInfo
from .rosetta_net_message_options import RosettaNetMessageOptions
from .rosetta_net_options import RosettaNetOptions


@JsonMap(
    {
        "rosetta_net_control_info": "RosettaNetControlInfo",
        "rosetta_net_message_options": "RosettaNetMessageOptions",
        "rosetta_net_options": "RosettaNetOptions",
    }
)
class RosettaNetPartnerInfo(BaseModel):
    """RosettaNetPartnerInfo

    :param rosetta_net_control_info: rosetta_net_control_info, defaults to None
    :type rosetta_net_control_info: RosettaNetControlInfo, optional
    :param rosetta_net_message_options: rosetta_net_message_options, defaults to None
    :type rosetta_net_message_options: RosettaNetMessageOptions, optional
    :param rosetta_net_options: rosetta_net_options, defaults to None
    :type rosetta_net_options: RosettaNetOptions, optional
    """

    def __init__(
        self,
        rosetta_net_control_info: RosettaNetControlInfo = SENTINEL,
        rosetta_net_message_options: RosettaNetMessageOptions = SENTINEL,
        rosetta_net_options: RosettaNetOptions = SENTINEL,
        **kwargs,
    ):
        """RosettaNetPartnerInfo

        :param rosetta_net_control_info: rosetta_net_control_info, defaults to None
        :type rosetta_net_control_info: RosettaNetControlInfo, optional
        :param rosetta_net_message_options: rosetta_net_message_options, defaults to None
        :type rosetta_net_message_options: RosettaNetMessageOptions, optional
        :param rosetta_net_options: rosetta_net_options, defaults to None
        :type rosetta_net_options: RosettaNetOptions, optional
        """
        if rosetta_net_control_info is not SENTINEL:
            self.rosetta_net_control_info = self._define_object(
                rosetta_net_control_info, RosettaNetControlInfo
            )
        if rosetta_net_message_options is not SENTINEL:
            self.rosetta_net_message_options = self._define_object(
                rosetta_net_message_options, RosettaNetMessageOptions
            )
        if rosetta_net_options is not SENTINEL:
            self.rosetta_net_options = self._define_object(
                rosetta_net_options, RosettaNetOptions
            )
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap(
    {
        "enable_archiving": "enableArchiving",
        "inbound_directory": "inboundDirectory",
        "outbound_directory": "outboundDirectory",
    }
)
class PartnerArchiving(BaseModel):
    """PartnerArchiving

    :param enable_archiving: enable_archiving, defaults to None
    :type enable_archiving: bool, optional
    :param inbound_directory: inbound_directory, defaults to None
    :type inbound_directory: str, optional
    :param outbound_directory: outbound_directory, defaults to None
    :type outbound_directory: str, optional
    """

    def __init__(
        self,
        enable_archiving: bool = SENTINEL,
        inbound_directory: str = SENTINEL,
        outbound_directory: str = SENTINEL,
        **kwargs
    ):
        """PartnerArchiving

        :param enable_archiving: enable_archiving, defaults to None
        :type enable_archiving: bool, optional
        :param inbound_directory: inbound_directory, defaults to None
        :type inbound_directory: str, optional
        :param outbound_directory: outbound_directory, defaults to None
        :type outbound_directory: str, optional
        """
        if enable_archiving is not SENTINEL:
            self.enable_archiving = enable_archiving
        if inbound_directory is not SENTINEL:
            self.inbound_directory = inbound_directory
        if outbound_directory is not SENTINEL:
            self.outbound_directory = outbound_directory
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"atom_id": "atomId", "include_bin": "includeBin", "log_date": "logDate"})
class AtomLog(BaseModel):
    """AtomLog

    :param atom_id: The ID of the Runtime., defaults to None
    :type atom_id: str, optional
    :param include_bin: If true, binary files are included in the download. The default is false., defaults to None
    :type include_bin: bool, optional
    :param log_date: The date of the logged events., defaults to None
    :type log_date: str, optional
    """

    def __init__(
        self,
        atom_id: str = SENTINEL,
        include_bin: bool = SENTINEL,
        log_date: str = SENTINEL,
        **kwargs
    ):
        """AtomLog

        :param atom_id: The ID of the Runtime., defaults to None
        :type atom_id: str, optional
        :param include_bin: If true, binary files are included in the download. The default is false., defaults to None
        :type include_bin: bool, optional
        :param log_date: The date of the logged events., defaults to None
        :type log_date: str, optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if include_bin is not SENTINEL:
            self.include_bin = include_bin
        if log_date is not SENTINEL:
            self.log_date = log_date
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .process_atom_attachment_expression import (
    ProcessAtomAttachmentExpression,
    ProcessAtomAttachmentExpressionGuard,
)
from .process_atom_attachment_simple_expression import (
    ProcessAtomAttachmentSimpleExpression,
)
from .process_atom_attachment_grouping_expression import (
    ProcessAtomAttachmentGroupingExpression,
)


@JsonMap({})
class ProcessAtomAttachmentQueryConfigQueryFilter(BaseModel):
    """ProcessAtomAttachmentQueryConfigQueryFilter

    :param expression: expression
    :type expression: ProcessAtomAttachmentExpression
    """

    def __init__(self, expression: ProcessAtomAttachmentExpression, **kwargs):
        """ProcessAtomAttachmentQueryConfigQueryFilter

        :param expression: expression
        :type expression: ProcessAtomAttachmentExpression
        """
        self.expression = ProcessAtomAttachmentExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ProcessAtomAttachmentQueryConfig(BaseModel):
    """ProcessAtomAttachmentQueryConfig

    :param query_filter: query_filter
    :type query_filter: ProcessAtomAttachmentQueryConfigQueryFilter
    """

    def __init__(
        self, query_filter: ProcessAtomAttachmentQueryConfigQueryFilter, **kwargs
    ):
        """ProcessAtomAttachmentQueryConfig

        :param query_filter: query_filter
        :type query_filter: ProcessAtomAttachmentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ProcessAtomAttachmentQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..environment_map_extension_user_defined_function import (
    EnvironmentMapExtensionUserDefinedFunctionService,
)
from ...models import (
    EnvironmentMapExtensionUserDefinedFunction,
    EnvironmentMapExtensionUserDefinedFunctionBulkResponse,
    EnvironmentMapExtensionUserDefinedFunctionBulkRequest,
)


class EnvironmentMapExtensionUserDefinedFunctionServiceAsync(
    EnvironmentMapExtensionUserDefinedFunctionService
):
    """
    Async Wrapper for EnvironmentMapExtensionUserDefinedFunctionServiceAsync
    """

    def create_environment_map_extension_user_defined_function(
        self, request_body: EnvironmentMapExtensionUserDefinedFunction = None
    ) -> Awaitable[Union[EnvironmentMapExtensionUserDefinedFunction, str]]:
        return to_async(super().create_environment_map_extension_user_defined_function)(
            request_body
        )

    def get_environment_map_extension_user_defined_function(
        self, id_: str
    ) -> Awaitable[Union[EnvironmentMapExtensionUserDefinedFunction, str]]:
        return to_async(super().get_environment_map_extension_user_defined_function)(
            id_
        )

    def update_environment_map_extension_user_defined_function(
        self, id_: str, request_body: EnvironmentMapExtensionUserDefinedFunction = None
    ) -> Awaitable[Union[EnvironmentMapExtensionUserDefinedFunction, str]]:
        return to_async(super().update_environment_map_extension_user_defined_function)(
            id_, request_body
        )

    def delete_environment_map_extension_user_defined_function(
        self, id_: str
    ) -> Awaitable[None]:
        return to_async(super().delete_environment_map_extension_user_defined_function)(
            id_
        )

    def bulk_environment_map_extension_user_defined_function(
        self, request_body: EnvironmentMapExtensionUserDefinedFunctionBulkRequest = None
    ) -> Awaitable[Union[EnvironmentMapExtensionUserDefinedFunctionBulkResponse, str]]:
        return to_async(super().bulk_environment_map_extension_user_defined_function)(
            request_body
        )




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..shared_communication_channel_component import (
    SharedCommunicationChannelComponentService,
)
from ...models import (
    SharedCommunicationChannelComponent,
    SharedCommunicationChannelComponentBulkResponse,
    SharedCommunicationChannelComponentBulkRequest,
    SharedCommunicationChannelComponentQueryResponse,
    SharedCommunicationChannelComponentQueryConfig,
)


class SharedCommunicationChannelComponentServiceAsync(
    SharedCommunicationChannelComponentService
):
    """
    Async Wrapper for SharedCommunicationChannelComponentServiceAsync
    """

    def create_shared_communication_channel_component(
        self, request_body: SharedCommunicationChannelComponent = None
    ) -> Awaitable[Union[str, SharedCommunicationChannelComponent]]:
        return to_async(super().create_shared_communication_channel_component)(
            request_body
        )

    def get_shared_communication_channel_component(
        self, id_: str
    ) -> Awaitable[Union[SharedCommunicationChannelComponent, str]]:
        return to_async(super().get_shared_communication_channel_component)(id_)

    def update_shared_communication_channel_component(
        self, id_: str, request_body: SharedCommunicationChannelComponent = None
    ) -> Awaitable[Union[SharedCommunicationChannelComponent, str]]:
        return to_async(super().update_shared_communication_channel_component)(
            id_, request_body
        )

    def delete_shared_communication_channel_component(
        self, id_: str
    ) -> Awaitable[None]:
        return to_async(super().delete_shared_communication_channel_component)(id_)

    def bulk_shared_communication_channel_component(
        self, request_body: SharedCommunicationChannelComponentBulkRequest = None
    ) -> Awaitable[Union[SharedCommunicationChannelComponentBulkResponse, str]]:
        return to_async(super().bulk_shared_communication_channel_component)(
            request_body
        )

    def query_shared_communication_channel_component(
        self, request_body: SharedCommunicationChannelComponentQueryConfig = None
    ) -> Awaitable[Union[SharedCommunicationChannelComponentQueryResponse, str]]:
        return to_async(super().query_shared_communication_channel_component)(
            request_body
        )

    def query_more_shared_communication_channel_component(
        self, request_body: str
    ) -> Awaitable[Union[SharedCommunicationChannelComponentQueryResponse, str]]:
        return to_async(super().query_more_shared_communication_channel_component)(
            request_body
        )




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .as2_communication_options import As2CommunicationOptions
from .disk_communication_options import DiskCommunicationOptions
from .ftp_communication_options import FtpCommunicationOptions
from .http_communication_options import HttpCommunicationOptions
from .mllp_communication_options import MllpCommunicationOptions
from .oftp_communication_options import OftpCommunicationOptions
from .sftp_communication_options import SftpCommunicationOptions


@JsonMap(
    {
        "as2_communication_options": "AS2CommunicationOptions",
        "disk_communication_options": "DiskCommunicationOptions",
        "ftp_communication_options": "FTPCommunicationOptions",
        "http_communication_options": "HTTPCommunicationOptions",
        "mllp_communication_options": "MLLPCommunicationOptions",
        "oftp_communication_options": "OFTPCommunicationOptions",
        "sftp_communication_options": "SFTPCommunicationOptions",
    }
)
class PartnerCommunication(BaseModel):
    """PartnerCommunication

    :param as2_communication_options: as2_communication_options, defaults to None
    :type as2_communication_options: As2CommunicationOptions, optional
    :param disk_communication_options: disk_communication_options, defaults to None
    :type disk_communication_options: DiskCommunicationOptions, optional
    :param ftp_communication_options: ftp_communication_options, defaults to None
    :type ftp_communication_options: FtpCommunicationOptions, optional
    :param http_communication_options: http_communication_options, defaults to None
    :type http_communication_options: HttpCommunicationOptions, optional
    :param mllp_communication_options: mllp_communication_options, defaults to None
    :type mllp_communication_options: MllpCommunicationOptions, optional
    :param oftp_communication_options: oftp_communication_options, defaults to None
    :type oftp_communication_options: OftpCommunicationOptions, optional
    :param sftp_communication_options: sftp_communication_options, defaults to None
    :type sftp_communication_options: SftpCommunicationOptions, optional
    """

    def __init__(
        self,
        as2_communication_options: As2CommunicationOptions = SENTINEL,
        disk_communication_options: DiskCommunicationOptions = SENTINEL,
        ftp_communication_options: FtpCommunicationOptions = SENTINEL,
        http_communication_options: HttpCommunicationOptions = SENTINEL,
        mllp_communication_options: MllpCommunicationOptions = SENTINEL,
        oftp_communication_options: OftpCommunicationOptions = SENTINEL,
        sftp_communication_options: SftpCommunicationOptions = SENTINEL,
        **kwargs,
    ):
        """PartnerCommunication

        :param as2_communication_options: as2_communication_options, defaults to None
        :type as2_communication_options: As2CommunicationOptions, optional
        :param disk_communication_options: disk_communication_options, defaults to None
        :type disk_communication_options: DiskCommunicationOptions, optional
        :param ftp_communication_options: ftp_communication_options, defaults to None
        :type ftp_communication_options: FtpCommunicationOptions, optional
        :param http_communication_options: http_communication_options, defaults to None
        :type http_communication_options: HttpCommunicationOptions, optional
        :param mllp_communication_options: mllp_communication_options, defaults to None
        :type mllp_communication_options: MllpCommunicationOptions, optional
        :param oftp_communication_options: oftp_communication_options, defaults to None
        :type oftp_communication_options: OftpCommunicationOptions, optional
        :param sftp_communication_options: sftp_communication_options, defaults to None
        :type sftp_communication_options: SftpCommunicationOptions, optional
        """
        if as2_communication_options is not SENTINEL:
            self.as2_communication_options = self._define_object(
                as2_communication_options, As2CommunicationOptions
            )
        if disk_communication_options is not SENTINEL:
            self.disk_communication_options = self._define_object(
                disk_communication_options, DiskCommunicationOptions
            )
        if ftp_communication_options is not SENTINEL:
            self.ftp_communication_options = self._define_object(
                ftp_communication_options, FtpCommunicationOptions
            )
        if http_communication_options is not SENTINEL:
            self.http_communication_options = self._define_object(
                http_communication_options, HttpCommunicationOptions
            )
        if mllp_communication_options is not SENTINEL:
            self.mllp_communication_options = self._define_object(
                mllp_communication_options, MllpCommunicationOptions
            )
        if oftp_communication_options is not SENTINEL:
            self.oftp_communication_options = self._define_object(
                oftp_communication_options, OftpCommunicationOptions
            )
        if sftp_communication_options is not SENTINEL:
            self.sftp_communication_options = self._define_object(
                sftp_communication_options, SftpCommunicationOptions
            )
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .edi_delimiter import EdiDelimiter
from .edi_segment_terminator import EdiSegmentTerminator


@JsonMap(
    {
        "composite_delimiter": "compositeDelimiter",
        "element_delimiter": "elementDelimiter",
        "filter_acknowledgements": "filterAcknowledgements",
        "segment_terminator": "segmentTerminator",
        "use_reconciliation_message": "useReconciliationMessage",
    }
)
class TradacomsOptions(BaseModel):
    """TradacomsOptions

    :param composite_delimiter: composite_delimiter, defaults to None
    :type composite_delimiter: EdiDelimiter, optional
    :param element_delimiter: element_delimiter, defaults to None
    :type element_delimiter: EdiDelimiter, optional
    :param filter_acknowledgements: filter_acknowledgements, defaults to None
    :type filter_acknowledgements: bool, optional
    :param segment_terminator: segment_terminator, defaults to None
    :type segment_terminator: EdiSegmentTerminator, optional
    :param use_reconciliation_message: use_reconciliation_message, defaults to None
    :type use_reconciliation_message: bool, optional
    """

    def __init__(
        self,
        composite_delimiter: EdiDelimiter = SENTINEL,
        element_delimiter: EdiDelimiter = SENTINEL,
        filter_acknowledgements: bool = SENTINEL,
        segment_terminator: EdiSegmentTerminator = SENTINEL,
        use_reconciliation_message: bool = SENTINEL,
        **kwargs,
    ):
        """TradacomsOptions

        :param composite_delimiter: composite_delimiter, defaults to None
        :type composite_delimiter: EdiDelimiter, optional
        :param element_delimiter: element_delimiter, defaults to None
        :type element_delimiter: EdiDelimiter, optional
        :param filter_acknowledgements: filter_acknowledgements, defaults to None
        :type filter_acknowledgements: bool, optional
        :param segment_terminator: segment_terminator, defaults to None
        :type segment_terminator: EdiSegmentTerminator, optional
        :param use_reconciliation_message: use_reconciliation_message, defaults to None
        :type use_reconciliation_message: bool, optional
        """
        if composite_delimiter is not SENTINEL:
            self.composite_delimiter = self._define_object(
                composite_delimiter, EdiDelimiter
            )
        if element_delimiter is not SENTINEL:
            self.element_delimiter = self._define_object(
                element_delimiter, EdiDelimiter
            )
        if filter_acknowledgements is not SENTINEL:
            self.filter_acknowledgements = filter_acknowledgements
        if segment_terminator is not SENTINEL:
            self.segment_terminator = self._define_object(
                segment_terminator, EdiSegmentTerminator
            )
        if use_reconciliation_message is not SENTINEL:
            self.use_reconciliation_message = use_reconciliation_message
        self._kwargs = kwargs




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"component_id": "componentId"})
class SharedCommunicationChannel(BaseModel):
    """SharedCommunicationChannel

    :param component_id: component_id, defaults to None
    :type component_id: str, optional
    """

    def __init__(self, component_id: str = SENTINEL, **kwargs):
        """SharedCommunicationChannel

        :param component_id: component_id, defaults to None
        :type component_id: str, optional
        """
        if component_id is not SENTINEL:
            self.component_id = component_id
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..atom_connection_field_extension_summary import (
    AtomConnectionFieldExtensionSummaryService,
)
from ...models import (
    AtomConnectionFieldExtensionSummaryQueryResponse,
    AtomConnectionFieldExtensionSummaryQueryConfig,
)


class AtomConnectionFieldExtensionSummaryServiceAsync(
    AtomConnectionFieldExtensionSummaryService
):
    """
    Async Wrapper for AtomConnectionFieldExtensionSummaryServiceAsync
    """

    def query_atom_connection_field_extension_summary(
        self, request_body: AtomConnectionFieldExtensionSummaryQueryConfig = None
    ) -> Awaitable[Union[AtomConnectionFieldExtensionSummaryQueryResponse, str]]:
        return to_async(super().query_atom_connection_field_extension_summary)(
            request_body
        )

    def query_more_atom_connection_field_extension_summary(
        self, request_body: str
    ) -> Awaitable[Union[AtomConnectionFieldExtensionSummaryQueryResponse, str]]:
        return to_async(super().query_more_atom_connection_field_extension_summary)(
            request_body
        )




from __future__ import annotations
from enum import Enum
from typing import List, TYPE_CHECKING
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL

if TYPE_CHECKING:
    from .execution_count_account_expression import ExecutionCountAccountExpression, ExecutionCountAccountExpressionGuard
from .execution_count_account_simple_expression import (
    ExecutionCountAccountSimpleExpression,
)

class ExecutionCountAccountGroupingExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar AND: "and"
    :vartype AND: str
    :cvar OR: "or"
    :vartype OR: str
    """

    AND = "and"
    OR = "or"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                ExecutionCountAccountGroupingExpressionOperator._member_map_.values(),
            )
        )

@JsonMap({"nested_expression": "nestedExpression"})
class ExecutionCountAccountGroupingExpression(BaseModel):
    """ExecutionCountAccountGroupingExpression

    :param nested_expression: nested_expression, defaults to None
    :type nested_expression: List[ExecutionCountAccountExpression], optional
    :param operator: operator
    :type operator: ExecutionCountAccountGroupingExpressionOperator
    """

    def __init__(
        self,
        operator: ExecutionCountAccountGroupingExpressionOperator,
        nested_expression: List[ExecutionCountAccountExpression] = SENTINEL,
        **kwargs,
    ):
        """ExecutionCountAccountGroupingExpression

        :param nested_expression: nested_expression, defaults to None
        :type nested_expression: List[ExecutionCountAccountExpression], optional
        :param operator: operator
        :type operator: ExecutionCountAccountGroupingExpressionOperator
        """
        if nested_expression is not SENTINEL:
            self.nested_expression = self._define_list(
                nested_expression, ExecutionCountAccountExpression
            )
        self.operator = self._enum_matching(
            operator, ExecutionCountAccountGroupingExpressionOperator.list(), "operator"
        )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..account_group_account import AccountGroupAccountService
from ...models import (
    AccountGroupAccount,
    AccountGroupAccountQueryResponse,
    AccountGroupAccountQueryConfig,
)


class AccountGroupAccountServiceAsync(AccountGroupAccountService):
    """
    Async Wrapper for AccountGroupAccountServiceAsync
    """

    def create_account_group_account(
        self, request_body: AccountGroupAccount = None
    ) -> Awaitable[Union[AccountGroupAccount, str]]:
        return to_async(super().create_account_group_account)(request_body)

    def query_account_group_account(
        self, request_body: AccountGroupAccountQueryConfig = None
    ) -> Awaitable[Union[AccountGroupAccountQueryResponse, str]]:
        return to_async(super().query_account_group_account)(request_body)

    def query_more_account_group_account(
        self, request_body: str
    ) -> Awaitable[Union[AccountGroupAccountQueryResponse, str]]:
        return to_async(super().query_more_account_group_account)(request_body)

    def delete_account_group_account(self, id_: str) -> Awaitable[None]:
        return to_async(super().delete_account_group_account)(id_)




from __future__ import annotations
from enum import Enum
from typing import List, TYPE_CHECKING
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL

if TYPE_CHECKING:
    from .integration_pack_instance_expression import IntegrationPackInstanceExpression, IntegrationPackInstanceExpressionGuard
from .integration_pack_instance_simple_expression import (
    IntegrationPackInstanceSimpleExpression,
)

class IntegrationPackInstanceGroupingExpressionOperator(Enum):
    """An enumeration representing different categories.

    :cvar AND: "and"
    :vartype AND: str
    :cvar OR: "or"
    :vartype OR: str
    """

    AND = "and"
    OR = "or"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                IntegrationPackInstanceGroupingExpressionOperator._member_map_.values(),
            )
        )

@JsonMap({"nested_expression": "nestedExpression"})
class IntegrationPackInstanceGroupingExpression(BaseModel):
    """IntegrationPackInstanceGroupingExpression

    :param nested_expression: nested_expression, defaults to None
    :type nested_expression: List[IntegrationPackInstanceExpression], optional
    :param operator: operator
    :type operator: IntegrationPackInstanceGroupingExpressionOperator
    """

    def __init__(
        self,
        operator: IntegrationPackInstanceGroupingExpressionOperator,
        nested_expression: List[IntegrationPackInstanceExpression] = SENTINEL,
        **kwargs,
    ):
        """IntegrationPackInstanceGroupingExpression

        :param nested_expression: nested_expression, defaults to None
        :type nested_expression: List[IntegrationPackInstanceExpression], optional
        :param operator: operator
        :type operator: IntegrationPackInstanceGroupingExpressionOperator
        """
        if nested_expression is not SENTINEL:
            self.nested_expression = self._define_list(
                nested_expression, IntegrationPackInstanceExpression
            )
        self.operator = self._enum_matching(
            operator,
            IntegrationPackInstanceGroupingExpressionOperator.list(),
            "operator",
        )
        self._kwargs = kwargs




from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .publisher_packaged_components import PublisherPackagedComponents


class PublisherIntegrationPackInstallationType(Enum):
    """An enumeration representing different categories.

    :cvar SINGLE: "SINGLE"
    :vartype SINGLE: str
    :cvar MULTI: "MULTI"
    :vartype MULTI: str
    """

    SINGLE = "SINGLE"
    MULTI = "MULTI"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                PublisherIntegrationPackInstallationType._member_map_.values(),
            )
        )


class OperationType(Enum):
    """An enumeration representing different categories.

    :cvar ADD: "ADD"
    :vartype ADD: str
    :cvar DELETE: "DELETE"
    :vartype DELETE: str
    """

    ADD = "ADD"
    DELETE = "DELETE"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, OperationType._member_map_.values()))


@JsonMap(
    {
        "description": "Description",
        "publisher_packaged_components": "PublisherPackagedComponents",
        "id_": "id",
        "installation_type": "installationType",
        "operation_type": "operationType",
    }
)
class PublisherIntegrationPack(BaseModel):
    """PublisherIntegrationPack

    :param description: description
    :type description: str
    :param publisher_packaged_components: publisher_packaged_components, defaults to None
    :type publisher_packaged_components: PublisherPackagedComponents, optional
    :param id_: A unique ID assigned by the system to the integration pack., defaults to None
    :type id_: str, optional
    :param installation_type: The type of integration pack. Possible values: - SINGLE — single attachment  - MULTI — multiple attachment, defaults to None
    :type installation_type: PublisherIntegrationPackInstallationType, optional
    :param name: The name of the integration pack., defaults to None
    :type name: str, optional
    :param operation_type: Specifies the type of operation (ADD or DELETE) to perform when updating the packaged component to the integration pack. This field is mandatory for the Update operation and is not available for other operations, defaults to None
    :type operation_type: OperationType, optional
    """

    def __init__(
        self,
        description: str,
        publisher_packaged_components: PublisherPackagedComponents = SENTINEL,
        id_: str = SENTINEL,
        installation_type: PublisherIntegrationPackInstallationType = SENTINEL,
        name: str = SENTINEL,
        operation_type: OperationType = SENTINEL,
        **kwargs,
    ):
        """PublisherIntegrationPack

        :param description: description
        :type description: str
        :param publisher_packaged_components: publisher_packaged_components, defaults to None
        :type publisher_packaged_components: PublisherPackagedComponents, optional
        :param id_: A unique ID assigned by the system to the integration pack., defaults to None
        :type id_: str, optional
        :param installation_type: The type of integration pack. Possible values: - SINGLE — single attachment  - MULTI — multiple attachment, defaults to None
        :type installation_type: PublisherIntegrationPackInstallationType, optional
        :param name: The name of the integration pack., defaults to None
        :type name: str, optional
        :param operation_type: Specifies the type of operation (ADD or DELETE) to perform when updating the packaged component to the integration pack. This field is mandatory for the Update operation and is not available for other operations, defaults to None
        :type operation_type: OperationType, optional
        """
        self.description = description
        if publisher_packaged_components is not SENTINEL:
            self.publisher_packaged_components = self._define_object(
                publisher_packaged_components, PublisherPackagedComponents
            )
        if id_ is not SENTINEL:
            self.id_ = id_
        if installation_type is not SENTINEL:
            self.installation_type = self._enum_matching(
                installation_type,
                PublisherIntegrationPackInstallationType.list(),
                "installation_type",
            )
        if name is not SENTINEL:
            self.name = name
        if operation_type is not SENTINEL:
            self.operation_type = self._enum_matching(
                operation_type, OperationType.list(), "operation_type"
            )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..shared_server_information import SharedServerInformationService
from ...models import (
    SharedServerInformation,
    SharedServerInformationBulkResponse,
    SharedServerInformationBulkRequest,
)


class SharedServerInformationServiceAsync(SharedServerInformationService):
    """
    Async Wrapper for SharedServerInformationServiceAsync
    """

    def get_shared_server_information(
        self, id_: str
    ) -> Awaitable[Union[SharedServerInformation, str]]:
        return to_async(super().get_shared_server_information)(id_)

    def update_shared_server_information(
        self, id_: str, request_body: SharedServerInformation = None
    ) -> Awaitable[Union[SharedServerInformation, str]]:
        return to_async(super().update_shared_server_information)(id_, request_body)

    def bulk_shared_server_information(
        self, request_body: SharedServerInformationBulkRequest = None
    ) -> Awaitable[Union[SharedServerInformationBulkResponse, str]]:
        return to_async(super().bulk_shared_server_information)(request_body)




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..node_offboard import NodeOffboardService
from ...models import NodeOffboard


class NodeOffboardServiceAsync(NodeOffboardService):
    """
    Async Wrapper for NodeOffboardServiceAsync
    """

    def create_node_offboard(
        self, request_body: NodeOffboard = None
    ) -> Awaitable[Union[NodeOffboard, str]]:
        return to_async(super().create_node_offboard)(request_body)




from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


@JsonMap({"atom_id": "atomId", "id_": "id", "process_id": "processId"})
class ProcessScheduleStatus(BaseModel):
    """ProcessScheduleStatus

    :param atom_id: A unique ID assigned by the system to the Runtime., defaults to None
    :type atom_id: str, optional
    :param enabled: If set to true, the schedules are in effect. Setting it to falsestops the schedules., defaults to None
    :type enabled: bool, optional
    :param id_: The object’s conceptual ID, which is synthesized from the process and Runtime IDs., defaults to None
    :type id_: str, optional
    :param process_id: A unique ID assigned by the system to the process., defaults to None
    :type process_id: str, optional
    """

    def __init__(
        self,
        atom_id: str = SENTINEL,
        enabled: bool = SENTINEL,
        id_: str = SENTINEL,
        process_id: str = SENTINEL,
        **kwargs
    ):
        """ProcessScheduleStatus

        :param atom_id: A unique ID assigned by the system to the Runtime., defaults to None
        :type atom_id: str, optional
        :param enabled: If set to true, the schedules are in effect. Setting it to falsestops the schedules., defaults to None
        :type enabled: bool, optional
        :param id_: The object’s conceptual ID, which is synthesized from the process and Runtime IDs., defaults to None
        :type id_: str, optional
        :param process_id: A unique ID assigned by the system to the process., defaults to None
        :type process_id: str, optional
        """
        if atom_id is not SENTINEL:
            self.atom_id = atom_id
        if enabled is not SENTINEL:
            self.enabled = enabled
        if id_ is not SENTINEL:
            self.id_ = id_
        if process_id is not SENTINEL:
            self.process_id = process_id
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .process_expression import ProcessExpression, ProcessExpressionGuard
from .process_simple_expression import ProcessSimpleExpression
from .process_grouping_expression import ProcessGroupingExpression


@JsonMap({})
class ProcessQueryConfigQueryFilter(BaseModel):
    """ProcessQueryConfigQueryFilter

    :param expression: expression
    :type expression: ProcessExpression
    """

    def __init__(self, expression: ProcessExpression, **kwargs):
        """ProcessQueryConfigQueryFilter

        :param expression: expression
        :type expression: ProcessExpression
        """
        self.expression = ProcessExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class ProcessQueryConfig(BaseModel):
    """ProcessQueryConfig

    :param query_filter: query_filter
    :type query_filter: ProcessQueryConfigQueryFilter
    """

    def __init__(self, query_filter: ProcessQueryConfigQueryFilter, **kwargs):
        """ProcessQueryConfig

        :param query_filter: query_filter
        :type query_filter: ProcessQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, ProcessQueryConfigQueryFilter
        )
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..execution_summary_record import ExecutionSummaryRecordService
from ...models import (
    ExecutionSummaryRecordQueryResponse,
    ExecutionSummaryRecordQueryConfig,
)


class ExecutionSummaryRecordServiceAsync(ExecutionSummaryRecordService):
    """
    Async Wrapper for ExecutionSummaryRecordServiceAsync
    """

    def query_execution_summary_record(
        self, request_body: ExecutionSummaryRecordQueryConfig = None
    ) -> Awaitable[Union[ExecutionSummaryRecordQueryResponse, str]]:
        return to_async(super().query_execution_summary_record)(request_body)

    def query_more_execution_summary_record(
        self, request_body: str
    ) -> Awaitable[Union[ExecutionSummaryRecordQueryResponse, str]]:
        return to_async(super().query_more_execution_summary_record)(request_body)




from __future__ import annotations
from typing import Union
from .utils.one_of_base_model import OneOfBaseModel
from .execution_record_simple_expression import ExecutionRecordSimpleExpression
from .execution_record_grouping_expression import ExecutionRecordGroupingExpression


class ExecutionRecordExpressionGuard(OneOfBaseModel):
    class_list = {
        "ExecutionRecordSimpleExpression": ExecutionRecordSimpleExpression,
        "ExecutionRecordGroupingExpression": ExecutionRecordGroupingExpression,
    }


ExecutionRecordExpression = Union[
    ExecutionRecordSimpleExpression, ExecutionRecordGroupingExpression
]




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .audit_log_property import AuditLogProperty


@JsonMap(
    {
        "audit_log_property": "AuditLogProperty",
        "account_id": "accountId",
        "container_id": "containerId",
        "date_": "date",
        "document_id": "documentId",
        "type_": "type",
        "user_id": "userId",
    }
)
class AuditLog(BaseModel):
    """AuditLog

    :param audit_log_property: audit_log_property, defaults to None
    :type audit_log_property: List[AuditLogProperty], optional
    :param account_id: The account in which the action occurred., defaults to None
    :type account_id: str, optional
    :param action: The action type., defaults to None
    :type action: str, optional
    :param container_id: The ID of the Runtime, Runtime cluster, or Runtime cloud on which the action occurred., defaults to None
    :type container_id: str, optional
    :param date_: The date and time the action occurred., defaults to None
    :type date_: str, optional
    :param document_id: The ID assigned to the Audit Log record., defaults to None
    :type document_id: str, optional
    :param level: The severity level of the action: DEBUG, ERROR, INFO, or WARNING., defaults to None
    :type level: str, optional
    :param message: A descriptive message. Not all types of management actions have a message in their audit log entries., defaults to None
    :type message: str, optional
    :param modifier: The action type qualifier., defaults to None
    :type modifier: str, optional
    :param source: Where the action occurred: API, INTERNAL, MOBILE, UI, or UNKNOWN, defaults to None
    :type source: str, optional
    :param type_: The type of object on which the action occurred., defaults to None
    :type type_: str, optional
    :param user_id: The ID \(email address\) of the user who performed the action., defaults to None
    :type user_id: str, optional
    """

    def __init__(
        self,
        audit_log_property: List[AuditLogProperty] = SENTINEL,
        account_id: str = SENTINEL,
        action: str = SENTINEL,
        container_id: str = SENTINEL,
        date_: str = SENTINEL,
        document_id: str = SENTINEL,
        level: str = SENTINEL,
        message: str = SENTINEL,
        modifier: str = SENTINEL,
        source: str = SENTINEL,
        type_: str = SENTINEL,
        user_id: str = SENTINEL,
        **kwargs,
    ):
        """AuditLog

        :param audit_log_property: audit_log_property, defaults to None
        :type audit_log_property: List[AuditLogProperty], optional
        :param account_id: The account in which the action occurred., defaults to None
        :type account_id: str, optional
        :param action: The action type., defaults to None
        :type action: str, optional
        :param container_id: The ID of the Runtime, Runtime cluster, or Runtime cloud on which the action occurred., defaults to None
        :type container_id: str, optional
        :param date_: The date and time the action occurred., defaults to None
        :type date_: str, optional
        :param document_id: The ID assigned to the Audit Log record., defaults to None
        :type document_id: str, optional
        :param level: The severity level of the action: DEBUG, ERROR, INFO, or WARNING., defaults to None
        :type level: str, optional
        :param message: A descriptive message. Not all types of management actions have a message in their audit log entries., defaults to None
        :type message: str, optional
        :param modifier: The action type qualifier., defaults to None
        :type modifier: str, optional
        :param source: Where the action occurred: API, INTERNAL, MOBILE, UI, or UNKNOWN, defaults to None
        :type source: str, optional
        :param type_: The type of object on which the action occurred., defaults to None
        :type type_: str, optional
        :param user_id: The ID \(email address\) of the user who performed the action., defaults to None
        :type user_id: str, optional
        """
        if audit_log_property is not SENTINEL:
            self.audit_log_property = self._define_list(
                audit_log_property, AuditLogProperty
            )
        if account_id is not SENTINEL:
            self.account_id = account_id
        if action is not SENTINEL:
            self.action = action
        if container_id is not SENTINEL:
            self.container_id = container_id
        if date_ is not SENTINEL:
            self.date_ = date_
        if document_id is not SENTINEL:
            self.document_id = document_id
        if level is not SENTINEL:
            self.level = level
        if message is not SENTINEL:
            self.message = message
        if modifier is not SENTINEL:
            self.modifier = modifier
        if source is not SENTINEL:
            self.source = source
        if type_ is not SENTINEL:
            self.type_ = type_
        if user_id is not SENTINEL:
            self.user_id = user_id
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .query_filter import QueryFilter


@JsonMap({"query_filter": "QueryFilter"})
class ConnectionLicensingReport(BaseModel):
    """ConnectionLicensingReport

    :param query_filter: query_filter
    :type query_filter: QueryFilter
    """

    def __init__(self, query_filter: QueryFilter, **kwargs):
        """ConnectionLicensingReport

        :param query_filter: query_filter
        :type query_filter: QueryFilter
        """
        self.query_filter = self._define_object(query_filter, QueryFilter)
        self._kwargs = kwargs




from typing import Awaitable, Union
from .utils.to_async import to_async
from ..atom_as2_artifacts import AtomAs2ArtifactsService
from ...models import LogDownload, AtomAs2Artifacts


class AtomAs2ArtifactsServiceAsync(AtomAs2ArtifactsService):
    """
    Async Wrapper for AtomAs2ArtifactsServiceAsync
    """

    def create_atom_as2_artifacts(
        self, request_body: AtomAs2Artifacts = None
    ) -> Awaitable[Union[LogDownload, str]]:
        return to_async(super().create_atom_as2_artifacts)(request_body)




from enum import Enum
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL


class AuthorizationInformationQualifier(Enum):
    """An enumeration representing different categories.

    :cvar X12AUTHQUAL00: "X12AUTHQUAL_00"
    :vartype X12AUTHQUAL00: str
    :cvar X12AUTHQUAL01: "X12AUTHQUAL_01"
    :vartype X12AUTHQUAL01: str
    :cvar X12AUTHQUAL02: "X12AUTHQUAL_02"
    :vartype X12AUTHQUAL02: str
    :cvar X12AUTHQUAL03: "X12AUTHQUAL_03"
    :vartype X12AUTHQUAL03: str
    :cvar X12AUTHQUAL04: "X12AUTHQUAL_04"
    :vartype X12AUTHQUAL04: str
    :cvar X12AUTHQUAL05: "X12AUTHQUAL_05"
    :vartype X12AUTHQUAL05: str
    """

    X12AUTHQUAL00 = "X12AUTHQUAL_00"
    X12AUTHQUAL01 = "X12AUTHQUAL_01"
    X12AUTHQUAL02 = "X12AUTHQUAL_02"
    X12AUTHQUAL03 = "X12AUTHQUAL_03"
    X12AUTHQUAL04 = "X12AUTHQUAL_04"
    X12AUTHQUAL05 = "X12AUTHQUAL_05"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                AuthorizationInformationQualifier._member_map_.values(),
            )
        )


class InterchangeIdQualifier(Enum):
    """An enumeration representing different categories.

    :cvar X12IDQUAL01: "X12IDQUAL_01"
    :vartype X12IDQUAL01: str
    :cvar X12IDQUAL02: "X12IDQUAL_02"
    :vartype X12IDQUAL02: str
    :cvar X12IDQUAL03: "X12IDQUAL_03"
    :vartype X12IDQUAL03: str
    :cvar X12IDQUAL04: "X12IDQUAL_04"
    :vartype X12IDQUAL04: str
    :cvar X12IDQUAL07: "X12IDQUAL_07"
    :vartype X12IDQUAL07: str
    :cvar X12IDQUAL08: "X12IDQUAL_08"
    :vartype X12IDQUAL08: str
    :cvar X12IDQUAL09: "X12IDQUAL_09"
    :vartype X12IDQUAL09: str
    :cvar X12IDQUAL10: "X12IDQUAL_10"
    :vartype X12IDQUAL10: str
    :cvar X12IDQUAL11: "X12IDQUAL_11"
    :vartype X12IDQUAL11: str
    :cvar X12IDQUAL12: "X12IDQUAL_12"
    :vartype X12IDQUAL12: str
    :cvar X12IDQUAL13: "X12IDQUAL_13"
    :vartype X12IDQUAL13: str
    :cvar X12IDQUAL14: "X12IDQUAL_14"
    :vartype X12IDQUAL14: str
    :cvar X12IDQUAL15: "X12IDQUAL_15"
    :vartype X12IDQUAL15: str
    :cvar X12IDQUAL16: "X12IDQUAL_16"
    :vartype X12IDQUAL16: str
    :cvar X12IDQUAL17: "X12IDQUAL_17"
    :vartype X12IDQUAL17: str
    :cvar X12IDQUAL18: "X12IDQUAL_18"
    :vartype X12IDQUAL18: str
    :cvar X12IDQUAL19: "X12IDQUAL_19"
    :vartype X12IDQUAL19: str
    :cvar X12IDQUAL20: "X12IDQUAL_20"
    :vartype X12IDQUAL20: str
    :cvar X12IDQUAL21: "X12IDQUAL_21"
    :vartype X12IDQUAL21: str
    :cvar X12IDQUAL22: "X12IDQUAL_22"
    :vartype X12IDQUAL22: str
    :cvar X12IDQUAL23: "X12IDQUAL_23"
    :vartype X12IDQUAL23: str
    :cvar X12IDQUAL24: "X12IDQUAL_24"
    :vartype X12IDQUAL24: str
    :cvar X12IDQUAL25: "X12IDQUAL_25"
    :vartype X12IDQUAL25: str
    :cvar X12IDQUAL26: "X12IDQUAL_26"
    :vartype X12IDQUAL26: str
    :cvar X12IDQUAL27: "X12IDQUAL_27"
    :vartype X12IDQUAL27: str
    :cvar X12IDQUAL28: "X12IDQUAL_28"
    :vartype X12IDQUAL28: str
    :cvar X12IDQUAL29: "X12IDQUAL_29"
    :vartype X12IDQUAL29: str
    :cvar X12IDQUAL30: "X12IDQUAL_30"
    :vartype X12IDQUAL30: str
    :cvar X12IDQUAL31: "X12IDQUAL_31"
    :vartype X12IDQUAL31: str
    :cvar X12IDQUAL32: "X12IDQUAL_32"
    :vartype X12IDQUAL32: str
    :cvar X12IDQUAL33: "X12IDQUAL_33"
    :vartype X12IDQUAL33: str
    :cvar X12IDQUAL34: "X12IDQUAL_34"
    :vartype X12IDQUAL34: str
    :cvar X12IDQUALNR: "X12IDQUAL_NR"
    :vartype X12IDQUALNR: str
    :cvar X12IDQUALZZ: "X12IDQUAL_ZZ"
    :vartype X12IDQUALZZ: str
    """

    X12IDQUAL01 = "X12IDQUAL_01"
    X12IDQUAL02 = "X12IDQUAL_02"
    X12IDQUAL03 = "X12IDQUAL_03"
    X12IDQUAL04 = "X12IDQUAL_04"
    X12IDQUAL07 = "X12IDQUAL_07"
    X12IDQUAL08 = "X12IDQUAL_08"
    X12IDQUAL09 = "X12IDQUAL_09"
    X12IDQUAL10 = "X12IDQUAL_10"
    X12IDQUAL11 = "X12IDQUAL_11"
    X12IDQUAL12 = "X12IDQUAL_12"
    X12IDQUAL13 = "X12IDQUAL_13"
    X12IDQUAL14 = "X12IDQUAL_14"
    X12IDQUAL15 = "X12IDQUAL_15"
    X12IDQUAL16 = "X12IDQUAL_16"
    X12IDQUAL17 = "X12IDQUAL_17"
    X12IDQUAL18 = "X12IDQUAL_18"
    X12IDQUAL19 = "X12IDQUAL_19"
    X12IDQUAL20 = "X12IDQUAL_20"
    X12IDQUAL21 = "X12IDQUAL_21"
    X12IDQUAL22 = "X12IDQUAL_22"
    X12IDQUAL23 = "X12IDQUAL_23"
    X12IDQUAL24 = "X12IDQUAL_24"
    X12IDQUAL25 = "X12IDQUAL_25"
    X12IDQUAL26 = "X12IDQUAL_26"
    X12IDQUAL27 = "X12IDQUAL_27"
    X12IDQUAL28 = "X12IDQUAL_28"
    X12IDQUAL29 = "X12IDQUAL_29"
    X12IDQUAL30 = "X12IDQUAL_30"
    X12IDQUAL31 = "X12IDQUAL_31"
    X12IDQUAL32 = "X12IDQUAL_32"
    X12IDQUAL33 = "X12IDQUAL_33"
    X12IDQUAL34 = "X12IDQUAL_34"
    X12IDQUALNR = "X12IDQUAL_NR"
    X12IDQUALZZ = "X12IDQUAL_ZZ"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, InterchangeIdQualifier._member_map_.values())
        )


class SecurityInformationQualifier(Enum):
    """An enumeration representing different categories.

    :cvar X12SECQUAL00: "X12SECQUAL_00"
    :vartype X12SECQUAL00: str
    :cvar X12SECQUAL01: "X12SECQUAL_01"
    :vartype X12SECQUAL01: str
    """

    X12SECQUAL00 = "X12SECQUAL_00"
    X12SECQUAL01 = "X12SECQUAL_01"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, SecurityInformationQualifier._member_map_.values())
        )


class Testindicator(Enum):
    """An enumeration representing different categories.

    :cvar P: "P"
    :vartype P: str
    :cvar T: "T"
    :vartype T: str
    """

    P = "P"
    T = "T"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Testindicator._member_map_.values()))


@JsonMap(
    {
        "authorization_information": "authorizationInformation",
        "authorization_information_qualifier": "authorizationInformationQualifier",
        "component_element_separator": "componentElementSeparator",
        "interchange_id": "interchangeId",
        "interchange_id_qualifier": "interchangeIdQualifier",
        "security_information": "securityInformation",
        "security_information_qualifier": "securityInformationQualifier",
        "standard_identification": "standardIdentification",
    }
)
class IsaControlInfo(BaseModel):
    """IsaControlInfo

    :param ackrequested: ackrequested, defaults to None
    :type ackrequested: bool, optional
    :param authorization_information: authorization_information, defaults to None
    :type authorization_information: str, optional
    :param authorization_information_qualifier: authorization_information_qualifier, defaults to None
    :type authorization_information_qualifier: AuthorizationInformationQualifier, optional
    :param component_element_separator: component_element_separator, defaults to None
    :type component_element_separator: str, optional
    :param interchange_id: interchange_id, defaults to None
    :type interchange_id: str, optional
    :param interchange_id_qualifier: interchange_id_qualifier, defaults to None
    :type interchange_id_qualifier: InterchangeIdQualifier, optional
    :param security_information: security_information, defaults to None
    :type security_information: str, optional
    :param security_information_qualifier: security_information_qualifier, defaults to None
    :type security_information_qualifier: SecurityInformationQualifier, optional
    :param standard_identification: standard_identification, defaults to None
    :type standard_identification: str, optional
    :param testindicator: testindicator, defaults to None
    :type testindicator: Testindicator, optional
    :param version: version, defaults to None
    :type version: str, optional
    """

    def __init__(
        self,
        ackrequested: bool = SENTINEL,
        authorization_information: str = SENTINEL,
        authorization_information_qualifier: AuthorizationInformationQualifier = SENTINEL,
        component_element_separator: str = SENTINEL,
        interchange_id: str = SENTINEL,
        interchange_id_qualifier: InterchangeIdQualifier = SENTINEL,
        security_information: str = SENTINEL,
        security_information_qualifier: SecurityInformationQualifier = SENTINEL,
        standard_identification: str = SENTINEL,
        testindicator: Testindicator = SENTINEL,
        version: str = SENTINEL,
        **kwargs
    ):
        """IsaControlInfo

        :param ackrequested: ackrequested, defaults to None
        :type ackrequested: bool, optional
        :param authorization_information: authorization_information, defaults to None
        :type authorization_information: str, optional
        :param authorization_information_qualifier: authorization_information_qualifier, defaults to None
        :type authorization_information_qualifier: AuthorizationInformationQualifier, optional
        :param component_element_separator: component_element_separator, defaults to None
        :type component_element_separator: str, optional
        :param interchange_id: interchange_id, defaults to None
        :type interchange_id: str, optional
        :param interchange_id_qualifier: interchange_id_qualifier, defaults to None
        :type interchange_id_qualifier: InterchangeIdQualifier, optional
        :param security_information: security_information, defaults to None
        :type security_information: str, optional
        :param security_information_qualifier: security_information_qualifier, defaults to None
        :type security_information_qualifier: SecurityInformationQualifier, optional
        :param standard_identification: standard_identification, defaults to None
        :type standard_identification: str, optional
        :param testindicator: testindicator, defaults to None
        :type testindicator: Testindicator, optional
        :param version: version, defaults to None
        :type version: str, optional
        """
        if ackrequested is not SENTINEL:
            self.ackrequested = ackrequested
        if authorization_information is not SENTINEL:
            self.authorization_information = authorization_information
        if authorization_information_qualifier is not SENTINEL:
            self.authorization_information_qualifier = self._enum_matching(
                authorization_information_qualifier,
                AuthorizationInformationQualifier.list(),
                "authorization_information_qualifier",
            )
        if component_element_separator is not SENTINEL:
            self.component_element_separator = component_element_separator
        if interchange_id is not SENTINEL:
            self.interchange_id = interchange_id
        if interchange_id_qualifier is not SENTINEL:
            self.interchange_id_qualifier = self._enum_matching(
                interchange_id_qualifier,
                InterchangeIdQualifier.list(),
                "interchange_id_qualifier",
            )
        if security_information is not SENTINEL:
            self.security_information = security_information
        if security_information_qualifier is not SENTINEL:
            self.security_information_qualifier = self._enum_matching(
                security_information_qualifier,
                SecurityInformationQualifier.list(),
                "security_information_qualifier",
            )
        if standard_identification is not SENTINEL:
            self.standard_identification = standard_identification
        if testindicator is not SENTINEL:
            self.testindicator = self._enum_matching(
                testindicator, Testindicator.list(), "testindicator"
            )
        if version is not SENTINEL:
            self.version = version
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .map_extensions_function import MapExtensionsFunction


@JsonMap({"function": "Function"})
class MapExtensionsFunctions(BaseModel):
    """The definition of Map function steps used in the Map component. For detailed information about how to define Map functions in a request or response, see the topic [Environment Map Extension functions](/docs/APIs/PlatformAPI/Environment_Map_Extension_functions). You can use the Extended Functions attribute to define the following extensible map functions (supports standard and user-defined function types):

    - User Defined
    - Connector
    - Lookup
    - Date
    - Numeric
    - String
    - Custom Scripting
    - Property

    :param function: function, defaults to None
    :type function: List[MapExtensionsFunction], optional
    """

    def __init__(self, function: List[MapExtensionsFunction] = SENTINEL, **kwargs):
        """The definition of Map function steps used in the Map component. For detailed information about how to define Map functions in a request or response, see the topic [Environment Map Extension functions](/docs/APIs/PlatformAPI/Environment_Map_Extension_functions). You can use the Extended Functions attribute to define the following extensible map functions (supports standard and user-defined function types):

        - User Defined
        - Connector
        - Lookup
        - Date
        - Numeric
        - String
        - Custom Scripting
        - Property

        :param function: function, defaults to None
        :type function: List[MapExtensionsFunction], optional
        """
        if function is not SENTINEL:
            self.function = self._define_list(function, MapExtensionsFunction)
        self._kwargs = kwargs




from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .utils.sentinel import SENTINEL
from .execution_record_expression import (
    ExecutionRecordExpression,
    ExecutionRecordExpressionGuard,
)
from .execution_record_simple_expression import ExecutionRecordSimpleExpression
from .execution_record_grouping_expression import ExecutionRecordGroupingExpression


@JsonMap({})
class ExecutionRecordQueryConfigQueryFilter(BaseModel):
    """ExecutionRecordQueryConfigQueryFilter

    :param expression: expression
    :type expression: ExecutionRecordExpression
    """

    def __init__(self, expression: ExecutionRecordExpression, **kwargs):
        """ExecutionRecordQueryConfigQueryFilter

        :param expression: expression
        :type expression: ExecutionRecordExpression
        """
        self.expression = ExecutionRecordExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"field_name": "fieldName", "sort_order": "sortOrder"})
class SortField(BaseModel):
    """SortField

    :param field_name: field_name, defaults to None
    :type field_name: str, optional
    :param sort_order: sort_order, defaults to None
    :type sort_order: str, optional
    """

    def __init__(
        self, field_name: str = SENTINEL, sort_order: str = SENTINEL, **kwargs
    ):
        """SortField

        :param field_name: field_name, defaults to None
        :type field_name: str, optional
        :param sort_order: sort_order, defaults to None
        :type sort_order: str, optional
        """
        if field_name is not SENTINEL:
            self.field_name = field_name
        if sort_order is not SENTINEL:
            self.sort_order = sort_order
        self._kwargs = kwargs


@JsonMap({"sort_field": "sortField"})
class QuerySort(BaseModel):
    """QuerySort

    :param sort_field: sort_field
    :type sort_field: List[SortField]
    """

    def __init__(self, sort_field: List[SortField], **kwargs):
        """QuerySort

        :param sort_field: sort_field
        :type sort_field: List[SortField]
        """
        self.sort_field = self._define_list(sort_field, SortField)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter", "query_sort": "QuerySort"})
class ExecutionRecordQueryConfig(BaseModel):
    """ExecutionRecordQueryConfig

    :param query_filter: query_filter
    :type query_filter: ExecutionRecordQueryConfigQueryFilter
    :param query_sort: query_sort, defaults to None
    :type query_sort: QuerySort, optional
    """

    def __init__(
        self,
        query_filter: ExecutionRecordQueryConfigQueryFilter,
        query_sort: QuerySort = SENTINEL,
        **kwargs,
    ):
        """ExecutionRecordQueryConfig

        :param query_filter: query_filter
        :type query_filter: ExecutionRecordQueryConfigQueryFilter
        :param query_sort: query_sort, defaults to None
        :type query_sort: QuerySort, optional
        """
        self.query_filter = self._define_object(
            query_filter, ExecutionRecordQueryConfigQueryFilter
        )
        if query_sort is not SENTINEL:
            self.query_sort = self._define_object(query_sort, QuerySort)
        self._kwargs = kwargs




from __future__ import annotations
from .utils.json_map import JsonMap
from .utils.base_model import BaseModel
from .deployment_expression import DeploymentExpression, DeploymentExpressionGuard
from .deployment_simple_expression import DeploymentSimpleExpression
from .deployment_grouping_expression import DeploymentGroupingExpression


@JsonMap({})
class DeploymentQueryConfigQueryFilter(BaseModel):
    """DeploymentQueryConfigQueryFilter

    :param expression: expression
    :type expression: DeploymentExpression
    """

    def __init__(self, expression: DeploymentExpression, **kwargs):
        """DeploymentQueryConfigQueryFilter

        :param expression: expression
        :type expression: DeploymentExpression
        """
        self.expression = DeploymentExpressionGuard.return_one_of(expression)
        self._kwargs = kwargs


@JsonMap({"query_filter": "QueryFilter"})
class DeploymentQueryConfig(BaseModel):
    """DeploymentQueryConfig

    :param query_filter: query_filter
    :type query_filter: DeploymentQueryConfigQueryFilter
    """

    def __init__(self, query_filter: DeploymentQueryConfigQueryFilter, **kwargs):
        """DeploymentQueryConfig

        :param query_filter: query_filter
        :type query_filter: DeploymentQueryConfigQueryFilter
        """
        self.query_filter = self._define_object(
            query_filter, DeploymentQueryConfigQueryFilter
        )
        self._kwargs = kwargs
